// Handler for custom URL scheme
function handleOpenURL(url) {

    var $injector = angular.element(document.body).injector();

    $injector.invoke(["$state", "session", "appConstants", "appValues", "$ionicLoading", "$timeout", "$log", "$translate", "PlatformCheck", function($state, session, appConstants, appValues, $ionicLoading, $timeout, $log, $translate, PlatformCheck) {

        $translate.onReady().then(function() {
            $ionicLoading.show({
                template: $translate.instant('GENERAL.VERIFYING_MESSAGE')
            });
        });
        var token = url.substring(url.indexOf('=') + 1);
        appValues.verificationToken = token;

        var credentials = {
            token: token,
            appId: appConstants.app.appId
        }

        if (!token) {
            $ionicLoading.hide();
            $state.go("auth.verificationError");
            return;
        }


        var errorMessage = '';
        var authentication = session.verifyLogin(credentials);
        authentication.success(function(data, status) {
            $timeout(function() {
                $log.debug("Verification (" + status + "): Verified!");
                $ionicLoading.hide();

                // TODO CHECK IF AUTOLOAD
                if (PlatformCheck.isDevice()) {
                    $state.go("auth.enablePushNotifications");
                } else {
                    $state.go("multi.eventList");
                }
            }, appConstants.app.debugging.standardTimeoutDelay);

        }).error(function(response) {
            errorMessage = response.message;
            $log.debug("Error: " + response.message);
            $ionicLoading.hide();
            $state.go("auth.verificationError");
        });

    }]);


}

var app = angular.module('app', ['ionic', 'app.controllers', 'app.services', 'ngSanitize', 'app.directives', 'ngCordova', 'breeze.angular', 'jett.ionic.filter.bar', 'ionic-toast', 'toaster', 'ngAnimate', 'ngIOS9UIWebViewPatch', 'ngMap', 'pascalprecht.translate', 'smoothScroll', 'ImgCache', 'ja.qr', 'matchMedia']);

app.constant('_',
window._
);
app.run(['$rootScope', 'PushService', '$ionicPlatform', 'session', '$log', '$cacheFactory', '$window', 'appConfiguration', 'appConstants', 'PlatformCheck', '$translate', 'AnalyticsService', '$cordovaDialogs', 'ImgCache', 'CustomColorService', 'ObjectService', 'eventService', 'datacontext', '$ionicModal', '$timeout', function($rootScope, PushService, $ionicPlatform, session, $log, $cacheFactory, $window, appConfiguration, appConstants, PlatformCheck, $translate, AnalyticsService, $cordovaDialogs, ImgCache, CustomColorService, ObjectService, eventService, datacontext, $ionicModal, $timeout) {



    $ionicPlatform.ready(function() {

        ImgCache.$init();

        window.open = cordova.InAppBrowser.open;

        if (appConstants.app.isContainerApp) {

            var currentEvent = ObjectService.getSimpleObject(e.apiContentTypes.Event, datacontext.getCachedEventInformation(eventService.getId()));
            
            var template = '<div class="modal image-modal blackBackground"><ion-pane class="blackBackground"><img img-cache ic-src="' + currentEvent.eventSplashScreenImageUrl + '" class="fullscreen-image" /></ion-pane></div>'
            //$rootScope.modal = $ionicModal.fromTemplate('<div class="modal image-modal blackBackground"><ion-pane class="blackBackground"><img ng-src="../img/demo_splash.png" class="fullscreen-image"/></ion-pane></div>', {
            $rootScope.modal = $ionicModal.fromTemplate(template, {
                scope: $rootScope,
                animation: 'none'
            });

            $rootScope.openModal = function () {
                $rootScope.modal.show();
            };

            $rootScope.closeModal = function () {
                $rootScope.modal.hide();
            };

            //Cleanup the modal when we're done with it!
            $rootScope.$on('$destroy', function () {
                $rootScope.modal.remove();
            });
            // Execute action on hide modal
            $rootScope.$on('modal.hide', function () {
                // Execute action
            });
            // Execute action on remove modal
            $rootScope.$on('modal.removed', function () {
                // Execute action
            });
            $rootScope.$on('modal.shown', function () {
                //console.log('Modal is shown!');
            });

            //console.log("logged text: " + currentEvent.eventSplashScreenImageUrl);
            //console.log(currentEvent);
            if (currentEvent.eventSplashScreenImageUrl != "") {
                $rootScope.openModal();

                $timeout(function () {
                    //console.log("called");
                    $rootScope.closeModal();
                }, appConstants.app.eventSplashScreenDelay);
            }
            
        }


        

        logger = function(str, level) {

            switch (level) {
                case 1:
                    {
                        // LEVEL: INFO
                        // No logging implemented at the moment
                        var currentCacheSize = ImgCache.getCurrentSize();
                        var eventProperties = {
                                "Cache Size": currentCacheSize,
                                "Error Level": "Info",
                                "Error Message": str
                            }
                            //AnalyticsService.trackEvent("Cache", eventProperties);
                        break;
                    }
                case 2:
                    {
                        // LEVEL: WARNING

                        var currentCacheSize = ImgCache.getCurrentSize();
                        var eventProperties = {
                            "Cache Size": currentCacheSize,
                            "Error Level": "Warning",
                            "Error Message": str
                        }
                        AnalyticsService.trackEvent("Cache", eventProperties);

                        break;
                    }
                case 3:
                    {
                        // LEVEL: ERROR

                        var currentCacheSize = ImgCache.getCurrentSize();
                        var eventProperties = {
                            "Cache Size": currentCacheSize,
                            "Error Level": "Error",
                            "Error Message": str
                        }
                        AnalyticsService.trackEvent("Cache", eventProperties);

                        break;
                    }
            }

        };

        var localCacheDebugging = appConstants.app.debugging.localCacheDebugging;
        if (!localCacheDebugging) {
            ImgCache.overridables.log = logger;
        }




    });

    /*
     mixpanel.register_once({
     "appId": appConstants.app.appId
     });
     mixpanel.time_event("appTimeSpent");
     */

    if (!ionic.Platform.isWebView()) {
        AnalyticsService.trackEvent("App Open");
    }

    $ionicPlatform.on('resume', function() {
        AnalyticsService.startTimer("App Open");

    });

    $ionicPlatform.on('pause', function() {

        AnalyticsService.trackEvent("App Open");


    });

    $translate.onReady().then(function() {

        try {
            localStorage.test = 2;
        } catch (e) {
            //on private browsing mode...
            $cordovaDialogs.alert($translate.instant('GENERAL.PRIVATE_BROWSING_MODE.MESSAGE'), $translate.instant('GENERAL.PRIVATE_BROWSING_MODE.TITLE'));
        }

        // Check if IE
        var IEversion = detectIE();

        if (IEversion !== false) {
            alert($translate.instant('GENERAL.BROWSER_NOT_SUPPORTED'));

        } else {
            //alert("NOT IE");
        }


    });


    /**
     * detect IE
     * returns version of IE or false, if browser is not Internet Explorer
     */
    function detectIE() {
        var ua = window.navigator.userAgent;

        // Test values; Uncomment to check result …

        // IE 10
        // ua = 'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0)';

        // IE 11
        // ua = 'Mozilla/5.0 (Windows NT 6.3; Trident/7.0; rv:11.0) like Gecko';

        // IE 12 / Spartan
        // ua = 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.71 Safari/537.36 Edge/12.0';

        // Edge (IE 12+)
        // ua = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/46.0.2486.0 Safari/537.36 Edge/13.10586';

        var msie = ua.indexOf('MSIE ');
        if (msie > 0) {
            // IE 10 or older => return version number
            return parseInt(ua.substring(msie + 5, ua.indexOf('.', msie)), 10);
        }

        var trident = ua.indexOf('Trident/');
        if (trident > 0) {
            // IE 11 => return version number
            var rv = ua.indexOf('rv:');
            return parseInt(ua.substring(rv + 3, ua.indexOf('.', rv)), 10);
        }

        var edge = ua.indexOf('Edge/');
        if (edge > 0) {
            // Edge (IE 12+) => return version number
            return parseInt(ua.substring(edge + 5, ua.indexOf('.', edge)), 10);
        }

        // other browser
        return false;
    }

    $ionicPlatform.ready(function() {


        CustomColorService.updateColorScheme();
        var currentEvent = ObjectService.getSimpleObject(e.apiContentTypes.Event, datacontext.getCachedEventInformation(eventService.getId()));
        
        $translate.use(currentEvent.language);
        moment.locale(currentEvent.language.substring(0,2));



        // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
        // for form inputs)
        if (window.cordova && window.cordova.plugins && window.cordova.plugins.Keyboard) {
            cordova.plugins.Keyboard.hideKeyboardAccessoryBar(false);
            cordova.plugins.Keyboard.disableScroll(true);
        }
        if (window.StatusBar) {
            // org.apache.cordova.statusbar required
            StatusBar.styleLightContent();
            StatusBar.styleDefault();
            StatusBar.backgroundColorByHexString("#2980b9"); // the blue color 
        }

        if (session.isAuthorized() && PlatformCheck.isDevice()) {
            PushService.registerDevice();
        }


        //This is for testing by Sijan-...
        //        var pushNotification;
        //
        //        document.addEventListener("deviceready", function () {
        //            pushNotification = window.plugins.pushNotification;
        //
        //        });


        //THIS IS INTERCEPTING ON RESUME...WE MUST SHOW THE LATEST PUSH MESSAGE WHEN ITS ON RESUME
        //IF USER OPENS APP WITHOUT CLICKING ON NOTIFICATION BROWSER,,THEN WE NEED TO SHOW THE LATEST PUSH MESSAGE OURSELF...
        //SO WE USE THIS METHOD!
        //        document.addEventListener("resume",function(){

        //            $rootScope.$broadcast("$cordovaPush:notificationReceived");
        //
        //
        //        });

        /*
        $rootScope.$on('$cordovaPush:notificationReceived', function (event, notification) {
            $log.debug("PUSH RECEIVED: " + JSON.stringify([notification]));
            if (ionic.Platform.isAndroid()) {
                PushService.handleAndroid(notification);
            } else if (ionic.Platform.isIOS()) {
                PushService.handleIOS(notification);
            }
        });*/


    });
}]);

/*
 app.config('appConstants', '$cacheFactory', '$ionicPlatform', [function ($appConstants, $cacheFactory, $ionicPlatform) {

 if (!$window.localStorage.getItem(appConfiguration.localStorageKeys.buildId) || ($window.localStorage.getItem(appConfiguration.localStorageKeys.buildId) == "")) {
 $window.localStorage.setItem($appConstants.localStorageKeys.buildId, $appConstants.app.buildId);
 } else if ($window.localStorage.getItem(appConfiguration.localStorageKeys.buildId) > $appConstants.app.buildId) {
 if (!ionic.Platform.isWebView()) {
 $cacheFactory('templates').removeAll();
 }
 $window.localStorage.setItem($appConstants.localStorageKeys.buildId, $appConstants.app.buildId);
 }
 }]);
 */

// Configure Localizing
app.config(['$translateProvider', 'appConstants', '$ionicConfigProvider', function($translateProvider, appConstants, $ionicConfigProvider) {

    $translateProvider.useStaticFilesLoader({
        prefix: 'js/language/',
        suffix: '.json'
    });

    $translateProvider.preferredLanguage(appConstants.app.preferredLanguage);

    moment.locale(appConstants.app.preferredLanguageShort);


}]);

app.config(['ImgCacheProvider', function(ImgCacheProvider) {
    // or more options at once
    ImgCacheProvider.setOptions({
        debug: true,
        usePersistentCache: true,
        connection: 'close'
    });

    ImgCacheProvider.manualInit = true;

    var _isDevice = false;

    var isDevice = function () {
        var cordovaCheck = document.URL.indexOf('http://') === -1 && document.URL.indexOf('https://') === -1;
        if (cordovaCheck) {
            _isDevice = true;
        } else {
            _isDevice = false;
        }

        return _isDevice;
    };

    if (!isDevice()) {
        ImgCacheProvider.disableCache(true);
    }
}]);

/* THis is for whitelisting the camera cordova file using ng-src */
app.config(function($compileProvider) {
    $compileProvider.imgSrcSanitizationWhitelist(/^\s*(https?|ftp|mailto|file|tel):/);
});

app.config(['$logProvider', 'appConstants', '$ionicConfigProvider', function($logProvider, appConstants, $ionicConfigProvider) {
    // Set Logging
    $logProvider.debugEnabled(appConstants.app.debugging.enableDebugLogging);
    $ionicConfigProvider.views.transition('fade-in');

}]);


app.config(function($stateProvider, $urlRouterProvider, $ionicConfigProvider, appConstants, $translateProvider) {

    //This is for the native scrolling for devices...
    //if (!ionic.Platform.isAndroid() || !ionic.Platform.isIOS()) $ionicConfigProvider.scrolling.jsScrolling(false);

    $translateProvider.useSanitizeValueStrategy('sanitizeParameters');
    $ionicConfigProvider.backButton.previousTitleText(false).text('');


    // Ionic uses AngularUI Router which uses the concept of states
    // Learn more here: https://github.com/angular-ui/ui-router
    // Set up the various states which the app can be in.
    // Each state's controller can be found in controllers.js
    $stateProvider
    //This is for the login one....
        .state('auth', {
        url: "/auth",
        abstract: true,
        templateUrl: "templates/auth.html?build=" + appConstants.app.buildId
    })

    /*THIS IS THE AUTHENTICATION PART */
    /*******************************/
    .state('auth.login', {
            url: '/login',
            views: {
                'login-view': {
                    templateUrl: 'templates/auth/signin_email.html?build=' + appConstants.app.buildId,
                    controller: 'SigninCtrl'
                }
            }
        })
        .state('auth.signinVerification', {
            url: '/signinVerification',
            views: {
                'login-view': {
                    templateUrl: 'templates/auth/signin_verification.html?build=' + appConstants.app.buildId,
                    controller: 'VerificationCtrl'
                }
            }
        })
        .state('auth.loginEmail', {
            url: '/loginEmail',
            views: {
                'login-view': {
                    templateUrl: 'templates/auth/loginEmail.html?build=' + appConstants.app.buildId,
                    controller: 'LoginCtrl'
                }
            }

        })
        .state('auth.pinVerification', {
            url: '/pinVerification',
            views: {
                'login-view': {
                    templateUrl: 'templates/auth/pinVerification.html?build=' + appConstants.app.buildId,
                    controller: 'LoginCtrl'
                }
            }

        })
        .state('auth.verificationSuccess', {
            url: '/verificationSuccess',
            views: {
                'login-view': {
                    templateUrl: 'templates/auth/verificationSuccess.html?build=' + appConstants.app.buildId,
                    controller: 'LoginCtrl'
                }
            }

        })
        .state('auth.verificationError', {
            url: '/verificationError',
            views: {
                'login-view': {
                    templateUrl: 'templates/auth/verificationError.html?build=' + appConstants.app.buildId,
                    controller: 'LoginCtrl'
                }
            }
        })
        .state('auth.sync', {
            url: '/sync',
            views: {
                'login-view': {
                    templateUrl: 'templates/auth/sync.html?build=' + appConstants.app.buildId,
                    controller: 'SyncCtrl'
                }
            },
            params: {
                forceProfileUpdate: null
            }
        })
        .state('auth.enablePushNotifications', {
            url: '/enablePushNotifications',
            views: {
                'login-view': {
                    templateUrl: 'templates/auth/enablePushNotifications.html?build=' + appConstants.app.buildId,
                    controller: 'EnablePushNotificationsController'
                }
            }
        })

    /*THIS IS THE MULTI EVENT PART */
    /*******************************/
    .state('multi', {
        url: "/multi",
        absctract: true,
        templateUrl: "templates/multi.html?build=" + appConstants.app.buildId
    })

    .state('multi.eventList', {
            url: "/multi/eventList",
            views: {
                'multi-view': {
                    templateUrl: "templates/multi/eventList.html",
                    controller: 'MultiEventCtrl',
                }
            },
            resolve: {
                events: function($stateParams, datacontext) {
                    return datacontext.getCachedEvents()
                }
            }
        })
        .state('multi.editProfile', {
            url: 'multi/editProfile',
            views: {
                'multi-view': {
                    templateUrl: 'templates/navigation/editProfile.html?build=' + appConstants.app.buildId,
                    controller: 'EditProfileCtrl'
                }
            }
        })
        .state('multi.updateProfile', {
            url: 'multi/updateProfile',
            views: {
                'multi-view': {
                    templateUrl: 'templates/multi/updateProfile.html?build=' + appConstants.app.buildId,
                    controller: 'UpdateProfileCtrl'
                }
            },
            params: {
                newProfile: null
            }
        })
        .state('multi.createProfile', {
            url: 'multi/createProfile',
            views: {
                'multi-view': {
                    templateUrl: 'templates/multi/createProfile.html?build=' + appConstants.app.buildId,
                    controller: 'CreateProfileCtrl'
                }
            },
            params: {
                eventCode: null,
                currentEvent: null
            }
        })


    /*THIS IS THE TABS PART AFTER THE LOGIN */
    /*******************************/

        // Each tab has its own nav history stack:


    /*THIS IS FOR THE TEST PART */
    /*******************************/
    .state('tabs.test', {
        url: '/test',
        views: {
            'tabs-test-view': {
                templateUrl: 'templates/navigation/testPage.html?build=' + appConstants.app.buildId,
                controller: 'PollCtrl'
            }
        }
    })

    /*THIS IS THE FEED PART */
    /*******************************/

    .state('tabs', {
        url: "/tabs",
        abstract: true,
        templateUrl: "templates/tabs.html?build=" + appConstants.app.buildId,
        controller: 'TabsController'
    })

    .state('tabs.feed', {
        url: '/feed',
        views: {
            'tabs-feed-view': {
                templateUrl: 'templates/navigation/feed.html?build=' + appConstants.app.buildId,
                controller: 'FeedCtrl'
            }
        }
    })

    .state('tabs.feedDetails', {
        url: '/feed/:postId',
        views: {
            'tabs-feed-view': {
                templateUrl: 'templates/navigation/feedDetails.html?build=' + appConstants.app.buildId,
                controller: 'FeedDetailsCtrl'
            }
        },
        resolve: {
            post: function($stateParams, datacontext) {
                return datacontext.getCachedPost($stateParams.postId)
            }
        }
    })

    .state('tabs.feedDetailsFromMenu', {
        url: '/feed/:postId',
        views: {
            'tabs-menu-view': {
                templateUrl: 'templates/navigation/feedDetails.html?build=' + appConstants.app.buildId,
                controller: 'FeedDetailsCtrl'
            }
        },
        resolve: {
            post: function($stateParams, datacontext) {
                return datacontext.getCachedPost($stateParams.postId)
            }
        }
    })

    .state('tabs.feedDetailsLikes', {
        url: '/feed/:postId/likes',
        views: {
            'tabs-feed-view': {
                templateUrl: 'templates/navigation/feedDetailsLikes.html?build=' + appConstants.app.buildId,
                controller: 'FeedDetailsLikesCtrl'
            }
        },
        resolve: {
            post: function($stateParams, datacontext) {
                return datacontext.getCachedPost($stateParams.postId)
            }
        }
    })

    .state('tabs.feedDetailsLikesFromMenu', {
        url: '/feed/:postId/likes',
        views: {
            'tabs-menu-view': {
                templateUrl: 'templates/navigation/feedDetailsLikes.html?build=' + appConstants.app.buildId,
                controller: 'FeedDetailsLikesCtrl'
            }
        },
        resolve: {
            post: function($stateParams, datacontext) {
                return datacontext.getCachedPost($stateParams.postId)
            }
        }
    })

    .state('tabs.participantDetailsFromFeed', {
        url: '/feed/participants/:participantId',
        views: {
            'tabs-feed-view': {
                templateUrl: 'templates/navigation/participantDetails.html?build=' + appConstants.app.buildId,
                controller: 'ParticipantProfileCtrl'
            }
        },
        resolve: {
            participant: function($stateParams, datacontext) {
                return datacontext.getCachedParticipant($stateParams.participantId)
            }
        }
    })

    .state('tabs.exhibitorDetailsFromFeed', {
        url: '/exhibitor/:exhibitorId',
        views: {
            'tabs-feed-view': {
                templateUrl: 'templates/navigation/exhibitorDetails.html?build=' + appConstants.app.buildId,
                controller: 'ExhibitorDetailCtrl'
            }
        },
        resolve: {
            exhibitor: function($stateParams, datacontext) {
                return datacontext.getCachedExhibitorProfile($stateParams.exhibitorId)
            }
        }
    })

    /*THIS IS THE AGENDA / SCHEDULE PART */
    /*******************************/
    .state('tabs.agenda', {
            url: '/agenda',
            views: {
                'tabs-agenda-view': {
                    templateUrl: 'templates/navigation/agenda.html?build=' + appConstants.app.buildId,
                    controller: 'AgendaCtrl'
                }
            }
        })
        .state('tabs.agendaDetails', {
            url: '/agenda/:agendaItemId',
            views: {
                'tabs-agenda-view': {
                    templateUrl: 'templates/navigation/agendaDetails.html?build=' + appConstants.app.buildId,
                    controller: 'AgendaDetailsCtrl'
                }
            },
            resolve: {
                agendaItem: function($stateParams, datacontext) {
                    return datacontext.getCachedScheduleItem($stateParams.agendaItemId)
                }
            }

        })
        .state('tabs.agendaDetailsFromSpeaker', {
            url: '/agendaFromSpeaker/:agendaItemId',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/agendaDetails.html?build=' + appConstants.app.buildId,
                    controller: 'AgendaDetailsCtrl'
                }
            },
            resolve: {
                agendaItem: function($stateParams, datacontext) {
                    return datacontext.getCachedScheduleItem($stateParams.agendaItemId)
                }
            }

        })
        .state('tabs.speakerDetails', {
            url: '/speaker/:speakerId',
            views: {
                'tabs-agenda-view': {
                    templateUrl: 'templates/navigation/speakerDetails.html?build=' + appConstants.app.buildId,
                    controller: 'SpeakerProfileCtrl'
                }
            },
            resolve: {
                speaker: function($stateParams, datacontext) {
                    return datacontext.getCachedSpeaker($stateParams.speakerId)
                }
            }

        })
        /* THIS IS THE PARTICIPANT PART * /
         /*******************************/

    .state('tabs.participants', {
            url: '/participants',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/participants.html?build=' + appConstants.app.buildId,
                    controller: 'ParticipantsCtrl'
                }
            }
        })
        .state('tabs.participantDetails', {
            url: '/participants/:participantId',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/participantDetails.html?build=' + appConstants.app.buildId,
                    controller: 'ParticipantProfileCtrl'
                }
            },
            resolve: {
                participant: function($stateParams, datacontext) {
                    return datacontext.getCachedParticipant($stateParams.participantId)
                }
            }
        })
        /* THIS IS THE MENU / ALL OTHER TABS WILL BE HERE * /
         /*******************************/
        .state('tabs.menu', {
            url: '/menu',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/menu.html?build=' + appConstants.app.buildId,
                    controller: 'MenuCtrl'
                }
            }
        })
        /* THIS IS THE EDIT PROFILE PAGE * /
         /*******************************/
        .state('tabs.editProfile', {
            url: 'menu/editProfile',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/editProfile.html?build=' + appConstants.app.buildId,
                    controller: 'EditProfileCtrl'
                }
            }
        })
        /* THIS IS THE POLLS PAGE * /
         /*******************************/
        .state('tabs.poll', {
            url: '/poll',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/polls.html?build=' + appConstants.app.buildId,
                    controller: 'PollCtrl'
                }
            }
        })
        .state('tabs.pollDetails', {
            url: '/pollDetails/:pollItemId',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/pollDetails.html?build=' + appConstants.app.buildId,
                    controller: 'PollDetailCtrl'
                }
            },
            resolve: {


                pollData: function($stateParams, PollService, $ionicLoading, $translate) {
                    //$translate.onReady().then(function () {
                    //$ionicLoading.show({
                    //    template: ". . ."
                    //});

                    return PollService.getPollDetails($stateParams.pollItemId)
                        //});
                }

            }
        })


    /* OPEN QUESTION PAGE*/
    /*******************************/
    .state('tabs.openQuestion', {
            url: '/openQuestion',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/openQuestion.html?build=' + appConstants.app.buildId,
                    controller: 'OpenQuestionCtrl'
                }
            }
        })
        .state('tabs.openQuestionDetails', {
            url: '/openQuestionDetails/:openQuestionItemId',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/openQuestionDetails.html?build=' + appConstants.app.buildId,
                    controller: 'OpenQuestionDetailsCtrl'
                }

            },
            resolve: {
                openQuestionData: function($stateParams, PollService, $ionicLoading, $translate) {
                    return PollService.getOpenQuestionDetails($stateParams.openQuestionItemId)

                }

            }
        }).state('tabs.openQuestionAnswerFeed', {
            url: '/openQuestionAnswerFeed/:openQuestionItemId',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/openQuestionAnswerFeed.html?build=' + appConstants.app.buildId,
                    controller: 'OpenQuestionAnswerFeedCtrl'
                }

            }
            /*,
             resolve: {
             openQuestionData: function ($stateParams, PollService, $ionicLoading, $translate) {
             return PollService.getOpenQuestionDetails($stateParams.openQuestionItemId)

             }

             }*/
        })


    /* THIS IS SCHEDULE PAGE * /
     /*******************************/

    .state('tabs.mySchedule', {
            url: '/mySchedule',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/mySchedule.html?build=' + appConstants.app.buildId,
                    controller: 'MyScheduleCtrl'
                }
            }
        })
        .state('tabs.myScheduleDetails', {
            url: '/mySchedule/:scheduleItemId',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/myScheduleDetails.html?build=' + appConstants.app.buildId,
                    controller: 'ScheduleProfileCtrl'
                }
            },
            resolve: {
                scheduleItem: function($stateParams, datacontext) {
                    return datacontext.getCachedScheduleItem($stateParams.scheduleItemId)
                }
            }

        })
        /*

        To be depreciated. Replaced by tabs.menuSpeakerDetails

                .state('tabs.mySpeakerDetails', {
                    url: '/mySpeaker/:speakerId',
                    views: {
                        'tabs-menu-view': {
                            templateUrl: 'templates/navigation/speakerDetails.html?build=' + appConstants.app.buildId,
                            controller: 'SpeakerProfileCtrl'
                        }
                    },
                    resolve: {
                        speaker: function ($stateParams, datacontext) {
                            return datacontext.getCachedSpeaker($stateParams.speakerId)
                        }

                    }

                })
                */

    /* THIS IS THE SPEAKER MODULE */
    /********************************/

    .state('tabs.menuSpeakers', {
        url: '/speakers',
        views: {
            'tabs-menu-view': {
                templateUrl: 'templates/navigation/speakers.html?build=' + appConstants.app.buildId,
                controller: 'SpeakersCtrl'
            }
        }
    })

    .state('tabs.menuSpeakerDetails', {
        url: '/mySpeaker/:speakerId',
        views: {
            'tabs-menu-view': {
                templateUrl: 'templates/navigation/speakerDetails.html?build=' + appConstants.app.buildId,
                controller: 'SpeakerProfileCtrl'
            }
        },
        resolve: {
            speaker: function($stateParams, datacontext) {
                return datacontext.getCachedSpeaker($stateParams.speakerId)
            }

        }

    })

        /* THIS IS THE IMAGE GALLERY MODULE */
    /********************************/

    .state('tabs.imageGallery', {
        url: '/imageGallery',
        views: {
            'tabs-menu-view': {
                templateUrl: 'templates/navigation/imageGallery.html?build=' + appConstants.app.buildId,
                controller: 'ImageGalleryCtrl'
            }
        }
    })

    /* THIS IS THE EXHIBITOR LIST PAGE * /
     /*******************************/
    .state('tabs.exhibitor', {
            url: '/exhibitor',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/exhibitor.html?build=' + appConstants.app.buildId,
                    controller: 'ExhibitorCtrl'
                }
            }
        })
        .state('tabs.exhibitorDetails', {
            url: '/exhibitor/:exhibitorId',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/exhibitorDetails.html?build=' + appConstants.app.buildId,
                    controller: 'ExhibitorDetailCtrl'
                }
            },
            resolve: {
                exhibitor: function($stateParams, datacontext) {
                    return datacontext.getCachedExhibitorProfile($stateParams.exhibitorId)
                }
            }
        })
        /* THIS IS THE VENUE MAP PAGE * /
         /*******************************/
        .state('tabs.venueMap', {
            url: '/venueMap/:venueMapId',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/venueMap.html?build=' + appConstants.app.buildId,
                    controller: 'VenueMapCtrl'
                }
            },
            resolve: {
                venueMap: function($stateParams, datacontext) {
                    return datacontext.getCachedInfoPage($stateParams.venueMapId)
                }
            }
        })
        /* THIS IS THE LOCATION MAP PAGE * /
         /*******************************/
        .state('tabs.locationMap', {
            url: '/locationMap',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/locationMap.html?build=' + appConstants.app.buildId,
                    controller: 'LocationMapCtrl'
                }
            },
            resolve: {
                locationMap: function(datacontext) {
                    return datacontext.getCachedLocationMap()
                }
            }
        })
        /* THIS IS THE INFOMATION PAGE * /
         /*******************************/
        .state('tabs.infoPage', {
            url: 'menu/infoPage/:infoPageId',
            views: {
                'tabs-menu-view': {
                    templateUrl: 'templates/navigation/infoPage.html?build=' + appConstants.app.buildId,
                    controller: 'InfoPageCtrl'
                }
            },
            resolve: {
                infoPage: function($stateParams, datacontext) {
                    return datacontext.getCachedInfoPage($stateParams.infoPageId)
                }
            }
        })

    /* THIS IS THE INFOMATION PAGE * /
    /*******************************/
    .state('tabs.aboutApp', {
        url: '/menu/aboutApp',
        views: {
            'tabs-menu-view': {
                templateUrl: 'templates/navigation/aboutEventEye.html?build=' + appConstants.app.buildId,
                controller: 'AboutAppCtrl'
            }
        },
        resolve: {
            infoPage: function($stateParams, datacontext) {
                return datacontext.getCachedInfoPage($stateParams.infoPageId)
            }
        }
    })


    /*THIS IS THE COMMENTED TABS PART */
    /*******************************/
    /*

     .state('tabs.myScheduleDetails', {
     url: '/mySchedule/:scheduleItemId',
     views: {
     'tabs-mySchedule-view': {
     templateUrl: 'templates/navigation/myScheduleDetails.html?build=' + appConstants.app.buildId',
     controller: 'ScheduleProfileCtrl'
     }
     },
     resolve: {
     scheduleItem: function ($stateParams, datacontext) {
     return datacontext.getCachedScheduleItem($stateParams.scheduleItemId)
     }
     }

     })
     .state('tabs.mySchedule', {
     url: '/mySchedule',
     views: {
     'tabs-mySchedule-view': {
     templateUrl: 'templates/navigation/mySchedule.html?build=' + appConstants.app.buildId',
     controller: 'MyScheduleCtrl'
     }
     }
     })



     .state('tabs.mySpeakerDetails', {
     url: '/mySpeaker/:speakerId',
     views: {
     'tabs-mySchedule-view': {
     templateUrl: 'templates/navigation/speakerDetails.html?build=' + appConstants.app.buildId,
     controller: 'SpeakerProfileCtrl'
     }
     },
     resolve: {
     speaker: function ($stateParams, datacontext) {
     return datacontext.getCachedSpeaker($stateParams.speakerId)
     }
     }


     */

    // if none of the above states are matched, use this as the fallback
    //$urlRouterProvider.otherwise('tab/feeds');

    $urlRouterProvider.otherwise(function($injector, $location) {
        $injector.invoke(['$state', 'eventService', 'session', 'datacontext', 'ObjectService',
            function($state, eventService, session, datacontext, ObjectService) {

                try {
                    if (eventService.getId() && (eventService.getId != '' && eventService != null)) {
                        //$log.debug("app.js: default loading defaultAuthorizedState");
                        $state.go(appConstants.app.defaultAuthorizedState);
                    } else if (!session.isAuthorized()) {
                        $state.go("multi.eventList");
                    } else {
                        //  $log.debug("app.js: session authorized");
                        datacontext.getEvents()
                            .then(function() {
                                var currentEvents = ObjectService
                                    .getSimpleObjectArray(e.apiContentTypes.Event, datacontext.getCachedEvents());

                                //   $log.debug("CURRENT EVENTS: " + currentEvents);
                                if (!currentEvents || currentEvents !== 1) {
                                    return;
                                }
                                //   $log.debug("Count: " + currentEvents.length + ". Object: " + currentEvents[0] + ". Autoload: " + currentEvents[0].autoloadEvent);
                                var targetEvent = currentEvents[0];

                                if (!targetEvent || !targetEvent) {
                                    return;
                                }

                                //  $log.debug("app.js: 1 event, data available, autoload true");
                                datacontext.clearData();

                                var token = session.getToken(),
                                    userParticipantId = session.getLoggedInParticipantId(),
                                    eventToken = targetEvent.authToken,
                                    targetEventId = targetEvent.id;

                                session.create('', token, true, userParticipantId, eventToken);

                                eventService.create(targetEventId);

                                $state.go("auth.sync");
                            })
                            .fail(function(error) {
                                //console.log('Unexpected error while getting current event', error);
                                $state.go("multi.eventList");
                            })
                    }
                } catch (error) {
                    //console.log('Unexpected error while getting current event', error);
                    $state.go("multi.eventList");
                }
            }
        ]);

    });
});