'use strict';

var app = angular.module('app');


app.factory("session", ['$window', '$http', '$state', 'appConstants', 'appConfiguration', 'datacontext', '$log', 'AnalyticsService', 'ImgCache', 'ObjectService', 'eventService', function ($window, $http, $state, appConstants, appConfiguration, datacontext, $log, AnalyticsService, ImgCache, ObjectService, eventService) {

    var getSessionString = function () {
        var sessionString = $window.localStorage.getItem(appConfiguration.localStorageKeys.auth.tokenKey);
        if (!sessionString) {
            sessionString = $window.sessionStorage.getItem(appConfiguration.localStorageKeys.auth.tokenKey);
        }
        return sessionString;
    };

    function getParsedSessionString()  {    
        var str = getSessionString();

            
        try {      
            return JSON.parse(str);    
        }    
        catch (e) {      
            return null;    
        }  
    };

    var getToken = function () {
        var sessionString = getParsedSessionString();
        return sessionString ? sessionString.token : null;
    }

    var getEventToken = function () {
        var sessionString = getParsedSessionString();
        return sessionString ? sessionString.eventToken : null;
    }

    var getCurrentEventId = function () {
        var sessionString = getParsedSessionString();
        return sessionString ? sessionString.currentEventId : null;
    }

    var getLoggedInParticipantId = function () {

        var currentEvent = datacontext.getCachedEventInformation(eventService.getId());
        return currentEvent.profileId;


        // var sessionString = getParsedSessionString();
        // return sessionString ? sessionString.userParticipantId : null;

    }

    var session = {
        create: function (userRole, token, saveToken, userParticipantId, eventToken) {
            $log.debug("userParticipantId: " + userParticipantId);
            var tokenObj = angular.toJson({
                value: userRole,
                token: token,
                eventToken: eventToken,
                userParticipantId: userParticipantId
            });
            if (saveToken) {
                $window.localStorage.setItem(appConfiguration.localStorageKeys.auth.tokenKey, tokenObj);
                $log.debug("Token from Local Storage: " + $window.localStorage.getItem(appConfiguration.localStorageKeys.auth.tokenKey));
                $log.debug("Token from Session Storage: " + $window.sessionStorage.getItem(appConfiguration.localStorageKeys.auth.tokenKey));
            } else
                $window.sessionStorage.setItem(appConfiguration.localStorageKeys.auth.tokenKey, tokenObj);
        },
        destroy: function () {

            datacontext.clearData();
            // Delete token
            $window.localStorage.removeItem(appConfiguration.localStorageKeys.auth.tokenKey);
            $window.sessionStorage.removeItem(appConfiguration.localStorageKeys.auth.tokenKey);
            // Clear Local Storage
            $window.localStorage.clear();
            // Clear Image Cache
            ImgCache.clearCache();

        },

        requestToken: function (credentials) {

            return session.verifyLogin(credentials);

            // if (credentials.email === appConstants.auth.demoLoginEmail) {
            //     return session.verifyLogin(credentials);
            // }

            // var registerUrl = appConfiguration.apiUrl + "account/multi-event/register?email=" + credentials.email + "&appId=" + credentials.appId;
            // var promise = $http.post(registerUrl);
            // promise.success(function (data, status) {
            //     $log.debug("STATUS: " + status);
            //     if (status === 200) {
            //         // Verification Email Sent
            //     }
            // }).error(function (data, status) {
            //     $log.debug("REQUEST TOKEN ERROR: " + data + " " + status);
            // });
            // return promise;
        },

        verifyLogin: function (credentials) {
            var token = credentials.token,
                appId = credentials.appId;

            if (ionic.Platform.isAndroid()) {
                token = encodeURIComponent(token)
            }

            var filteredEmail = credentials.email.replace(/[\s]/g, '');

            // CHANGES FOR PHONE VERIFICATION
            // http://ee-mobile-api-dev.azurewebsites.net/api/v1.2.3/account/multi-event/validate-phone?phone={phone}&appId=1024
            var url = (appConfiguration.apiUrl + "account/multi-event/validate-phone?" +
            "phone=" + encodeURIComponent(filteredEmail) + "&" +
            "appId=" + credentials.appId);

            // if (credentials.email === appConstants.auth.demoLoginEmail) {
            //     url = (appConfiguration.apiUrl + "account/multi-event/demo-login?appId=" + appConstants.app.appId);
            //     $log.debug(url);
            // }

            var promise = $http.get(url);

            promise.success(function (data, status) {
                if (status === 200) {
                    $log.debug("VERIFIED LOGIN!");
                    session.create('', data.authenticationToken, true, data.user.userId, ''); // Hardcoded values as roles is not implemented at this point

                    //console.log("Logging analytics");
                    //AnalyticsService.registerPeopleProperties("Signed In", {"Signed In": true, "Verification Method": "Link"});
                    // TODO: Go to sync or event list
                }
            }).error(function (data, status) {
                $log.debug("VERIFYLOGIN ERROR: " + data + " " + status);
            });

            return promise;
        },

        verifyPIN: function (credentials) {
            var url = (appConfiguration.apiUrl + "account/multi-event/validate-pin?" +
                "pin=" + credentials.pin + "&" +
                "email=" + credentials.email + "&" +
                "appId=" + credentials.appId);

            var promise = $http.get(url);
            promise.success(function (data, status) {
                if (status === 200) {
                    $log.debug("PIN Verification Successfull");
                    session.create('', data.authenticationToken, true, data.user.userId, '');

                    //console.log("Logging analytics");
                    //AnalyticsService.registerPeopleProperties("Signed In", {"Signed In": true, "Verification Method": "PIN"});

                }
            }).error(function (data, status) {
                $log.debug("PIN LOGIN ERROR: " + data + " " + status);
            });

            return promise;
        },

        verifyEventCode: function (eventCode) {
            var url = (appConfiguration.apiUrl + "event/by-code?" +
                "eventCode=" + eventCode);

            var promise = $http.get(url);
            promise.success(function (data, status) {
                if (status === 200) {
                    $log.debug("Event Code Verification Successful");

                }
            }).error(function (data, status) {
                $log.debug("PIN LOGIN ERROR: " + data + " " + status);
            });

            return promise;
        },

        createProfile: function (profileData) {

            var promise = $http.post((appConfiguration.apiUrl + "event/join-by-code"), profileData)
                .success(function (data, status) {
                    $log.debug("PROFILE CREATED");
                    return data;

                })
                .error(function (data, status) {
                    $log.debug("PROFILE CREATION ERROR: " + data + " " + status);
                    return;
                });

            return promise;
        },

        renewToken: function () {
            var url = appConfiguration.apiUrl + "auth/renew-token";

            var useEventToken = false;
            if (session.getEventToken() && session.getEventToken !== "") {
                useEventToken = true;
            }

            var oldToken = useEventToken ? session.getEventToken() : session.getToken();

            $log.debug("URL: " + url);
            $log.debug("PAYLOAD: " + '"' + oldToken + '"');

            var promise = $http.post(url, '"' + oldToken + '"');
            promise.success(function (data, status) {
                if (status === 200) {

                    // New token received
                    $log.debug("THIS IS THE RETURNED DATA: " + data);

                    // Add code to handle it here.
                    if (useEventToken) {
                        session.create('', session.getToken(), true, data.user.userId, data.authenticationToken);
                    } else {
                        session.create('', data.authenticationToken, true, data.user.userId, session.getEventToken());
                    }
                    $log.debug("Renewal of token complete: " + data + " " + status);
                }
            }).error(function (data, status) {
                // In case token renewal fails, user is not authenticated and session should be terminated.

                $log.debug("Fetching Token ERROR: " + data + " " + status);

            });

            return promise;
        },

        logout: function () {
            session.destroy();
            datacontext.destroyData();
            $state.go(appConstants.auth.loginState);
        },

        /*

         // Not needed?

         getRedirect: function () {
         var sessionString = $window.sessionStorage.getItem(appAuthConfig.redirectKey);
         if (sessionString) {
         return angular.fromJson(sessionString);
         }
         return null;
         },

         */


        getUserRole: function () {
            var token = $window.sessionStorage.getItem(appConfiguration.localStorageKeys.auth.tokenKey) || $window.localStorage.getItem(appConfiguration.localStorageKeys.auth.tokenKey);
            return angular.fromJson(token).value;
        },

        updateUserRole: function (newVal) {
            var source;
            var token = $window.localStorage.getItem(appConfiguration.localStorageKeys.auth.tokenKey);
            if (token)
                source = $window.localStorage;
            else {
                token = $window.sessionStorage.getItem(appConfiguration.localStorageKeys.auth.tokenKey);
                source = $window.sessionStorage;
            }

            if (!token)
                return;

            var jsObj = angular.fromJson(token);
            jsObj.value = newVal;
            source.setItem(appConfiguration.localStorageKeys.auth.tokenKey, angular.toJson(jsObj));
        },


        getToken: getToken,
        getEventToken: getEventToken,
        getCurrentEventId: getCurrentEventId,
        getLoggedInParticipantId: getLoggedInParticipantId,

        isAuthorized: function () {
            return !!getToken();
        },

        saveRedirect: function (stateObj) {
            $window.sessionStorage.setItem(appConstants.auth.redirectKey, angular.toJson(stateObj));
        },


    };

    return session;
}]);

// inject additional logic into httpProvider for adding X-Token header into all requests;
app.config(['$httpProvider', 'appConstants', function ($httpProvider, appConstants) {
    var interceptor = ['$injector', '$q', function ($injector, $q) {
        return {
            request: function (config) {
                delete $httpProvider.defaults.headers.post['Content-type'];
                var session = $injector.get('session');

                if ((config.url.indexOf("/auth/renew-token") >= 0) || (config.url.indexOf("hooks.zapier.com") >= 0) || (config.url.indexOf("/check") >= 0)) {
                    return config;
                }

                if (appConstants.app.debugging.useCustomToken === true) {
                    config.headers["X-ZUMO-AUTH"] = appConstants.app.debugging.customToken;
                } else {
                    if (session.isAuthorized()) {

                        var token = "";

                        if (session.getEventToken() && session.getEventToken != "") {
                            token = session.getEventToken();

                            //$log.debug("USING EVENT TOKEN: " + token);


                        } else {
                            token = session.getToken();
                            //$log.debug("USING TOKEN: " + token);
                        }


                        config.headers["X-ZUMO-AUTH"] = token;
                    }
                }
                // 'https://dev-eventeye.azure-mobile.net/api/v1.2.3/auth/renew-token')


                return config;
            },
            responseError: function (rejection) {

                var eventProperties = {}
                try {
                    eventProperties = {
                        "Request Endpoint": rejection.config.url,
                        "Request Method": rejection.config.method,
                        "Error code": rejection.status,
                        "Error description": rejection.statusText
                    };
                } catch (err) {

                }

                var analyticsService = $injector.get('AnalyticsService');
                analyticsService.trackEvent("HTTP Error", eventProperties);


                if (rejection.status === 401) {

                    //$log.debug("RESPONS ERROR 401 - RESPONS ERROR 401 - RESPONS ERROR 401 - RESPONS ERROR 401 - RESPONS ERROR 401")

                    var session = $injector.get('session');

                    var newTokenRequest = session.renewToken();

                    newTokenRequest.success(function (data, status) {

                        console.log("status: " + status);
                        if (status === 200) {

                            return;

                        } else {
                            var session = $injector.get('session');
                            session.logout();


                            return $q.reject(rejection);
                        }
                    }).error(function () {
                        // In case token renewal fails, user is not authenticated and session should be terminated.

                        var session = $injector.get('session');
                        session.logout();


                        return $q.reject(rejection);
                    });


                } else if (rejection.status === 403) {
                    // Terms not accepted
                    var payload = rejection.data;

                    if (payload.code == "INITIAL_TERMS_AGREEMENT_REQUIRED") {
                        /* console.log("403 error - initial terms");
                        var translate = $injector.get('$translate');
                        var ionicPopup = $injector.get('$ionicPopup');
                        translate.onReady().then(function () {
                            var myPopup = ionicPopup.show({
                                template: translate.instant('GENERAL.ERRORS.403.INITIAL_TERMS_AGREEMENT_REQUIRED.BODY'),
                                title: translate.instant('GENERAL.ERRORS.403.INITIAL_TERMS_AGREEMENT_REQUIRED.TITLE'),
                                buttons: [
                                    { 
                                        text: translate.instant('GENERAL.ERRORS.403.INITIAL_TERMS_AGREEMENT_REQUIRED.CANCEL_BUTTON'),
                                        type: 'button-assertive',
                                        onTap: function(e) {
                                            var session = $injector.get('session');
                                            session.logout();

                                            return $q.reject(rejection);
                                            myPopup.close();
                                        } 
                                    },
                                    {
                                        text: translate.instant('GENERAL.ERRORS.403.INITIAL_TERMS_AGREEMENT_REQUIRED.OK_BUTTON'),
                                        type: 'button-calm',
                                        onTap: function(e) {
                                            // Set accepted
                                            // Call PUT updateUserAcceptedTerms endpoint
                                            // if successful --> continue
                                            // if not successful --> signout
                                            var session = $injector.get('session');
                                            session.logout();

                                            return $q.reject(rejection);
                                        }
                                    }
                                ]
                            });
                        }); */

                        var session = $injector.get('session');
                        session.logout();

                        return $q.reject(rejection);


                    } else if (payload.code == "UPDATED_TERMS_AGREEMENT_REQUIRED") {
                        /* console.log("403 error - updated terms");

                        var translate = $injector.get('$translate');
                        var ionicPopup = $injector.get('$ionicPopup');
                        translate.onReady().then(function () {
                            var myPopup = ionicPopup.show({
                                template: translate.instant('GENERAL.ERRORS.403.INITIAL_TERMS_AGREEMENT_REQUIRED.BODY'),
                                title: translate.instant('GENERAL.ERRORS.403.INITIAL_TERMS_AGREEMENT_REQUIRED.TITLE'),
                                buttons: [{
                                        text: translate.instant('GENERAL.ERRORS.403.INITIAL_TERMS_AGREEMENT_REQUIRED.CANCEL_BUTTON'),
                                        type: 'button-assertive',
                                        onTap: function (e) {
                                            var session = $injector.get('session');
                                            session.logout();

                                            return $q.reject(rejection);
                                        }
                                    },
                                    {
                                        text: translate.instant('GENERAL.ERRORS.403.INITIAL_TERMS_AGREEMENT_REQUIRED.OK_BUTTON'),
                                        type: 'button-calm',
                                        onTap: function (e) {
                                            // Set accepted
                                            // Call PUT updateUserAcceptedTerms endpoint
                                            // if successful --> continue
                                            // if not successful --> signout
                                            var session = $injector.get('session');
                                            session.logout();

                                            return $q.reject(rejection);
                                        }
                                    }
                                ]
                            });
                        }); */

                        var session = $injector.get('session');
                        session.logout();

                        return $q.reject(rejection);

                    } else if (payload.code == "AUTHORIZATION_REQUIRED") {
                        //alert("authorization required");

                    } else if (payload.code == "ERROR_OCCURRED") {
                        //alert("error occurred");
                    }


                } else {
                    return $q.reject(rejection);
                }

            }
        };
    }];

    $httpProvider.interceptors.push(interceptor);
}]);


// checking user's authorization before change ui-route state
app.run(['$rootScope', '$state', '$stateParams', 'session', 'appConstants', '$ionicPlatform', '$window', 'appConfiguration',
    function ($rootScope, $state, $stateParams, session, appConstants, $ionicPlatform, $window, appConfiguration) {

        function gotoSignin() {
            var selectedEmail = $window.localStorage.getItem(appConfiguration.localStorageKeys.auth.lastUsedEmailKey);
            if (selectedEmail != undefined && selectedEmail && selectedEmail.length > 0) {
                /*$ionicHistory.nextViewOptions({
                    disableAnimate: true
                });*/
                $state.transitionTo("auth.signinVerification", {
                    reload: true
                });
            } else {
                $state.transitionTo("auth.login", {
                    reload: true
                });
            }



        }

        $rootScope.$on("$stateChangeStart", function (event, toState, toParams, fromState, fromParams) {

            var all = appConstants.auth.protectAllRoutes;
            if (all !== false) {

                // User is not authenticated
                if (!session.isAuthorized()) {

                    if ((/^auth.*/i).test(toState.name)) {
                        //gotoSignin();
                        return false;
                    }

                    session.saveRedirect({
                        state: toState.name,
                        params: toParams
                    });
                    event.preventDefault();

                    /*$state.transitionTo(appConstants.auth.loginState, {
                        reload: true
                    });*/
                    gotoSignin();
                }
                /*else if (toState.userRoles) {
                 if (!$rootScope.user)
                 return false;

                 if (toState.userRoles.indexOf($rootScope.user.userRole))
                 return true;
                 }*/
            }
            return true;
        });


    }
]);