controllers.controller('AgendaDetailsCtrl', ['$scope', '$ionicHistory', '$log', 'agendaItem', 'datacontext', 'ActionService', 'AnalyticsService', '$state', '$ionicHistory', function ($scope, $ionicHistory, $log, agendaItem, datacontext, ActionService, AnalyticsService, $state, $ionicHistory) {

    $scope.closeWindow = function () {
        $log.debug("Closing Window");
        $ionicHistory.goBack();
    }

    $scope.agendaItem = agendaItem;
    $scope.isShowAll = true; //This value is to show or not show the options menu...

    $scope.$on("$ionicView.beforeEnter", function() {
        // If navigation history is empty (ie. browser refresh) --> reset state to parent tab
        if ($ionicHistory.viewHistory().currentView.backViewId == null) {
            $ionicHistory.nextViewOptions({
                disableAnimate: true,
                disableBack: true
            });
            if ($state.current.name == "tabs.agendaDetails") {
                $state.go("tabs.agenda");
            } else {
                $state.go("tabs.menu");
            }
            
        }       
    });

    $scope.$on('$ionicView.enter', function (e) {
        
        // Analytics
        if (!ionic.Platform.isWebView()) {
            trackProfileView();
        } else {
            AnalyticsService.startTimer("Agenda Profile State Open");
        }

        //   $log.debug("SpeakerIds: " + agendaItem.speakerIds);
        $log.debug("Agenda Name:" + agendaItem.title);
        
        $log.debug("ACTION: " + agendaItem.actions);
        if (agendaItem.actions != undefined && agendaItem.actions && agendaItem.actions != "") {
            $scope.actions = JSON.parse(agendaItem.actions);
        }

        $scope.currentState = $state.current.name; 
        $scope.speakers = datacontext.getCachedSpeakers(agendaItem.speakerIds);

        
        
        

    });
    
    $scope.$on('$ionicView.leave', function (e) {
        if (ionic.Platform.isWebView()) {
            trackProfileView();
        }
    });
    
    function trackProfileView() {
        var eventProperties = {
            "State Name": "Agenda Details",
            "Session Id": $scope.agendaItem.id,
            "Session Title": $scope.agendaItem.title
        };
        AnalyticsService.trackEvent("Agenda Profile State Open", eventProperties);
    }
    
    /* THIS WILL SET THE HEIGHT OF EACH ITEM...*/
    $scope.setItemHeight = function (speakerItem) {
        //if the sub-heading is too long...set size big...
        var totalChars = 0;
        if (speakerItem.position != null) {
            totalChars += speakerItem.position.length;
        }
        if (speakerItem.companyName != null) {
            totalChars += speakerItem.companyName.length;
        }
        if (totalChars >= 40) //if subheader is more than 50 chars....then bigItem height....
            return '94px';
        else
            return '75px';
    }

    $scope.changeAttendingStatus = function (scheduleItem) {
        $log.debug("Changing Schedule Item for: " + scheduleItem.title);
        $log.debug("Current status: " + scheduleItem.isAttend);
        scheduleItem.isAttend = !scheduleItem.isAttend;
        datacontext.saveNewAttendingStatus(scheduleItem);

    }
    
    $scope.actionClicked = function (action) { 
        ActionService.performAction(action);
    }

    $scope.showEndTime = function(startTime, endTime) {
        if (moment(endTime).isAfter(startTime)) {
            return false;
        } else {
            return true;
        }
    }

}]).directive('animatesvg', function () {
    return {
        restrict: 'A',
        compile: function () {
            var config = {
                    animation: [
                        {
                            el: 'path:nth-child(2)',
                            animProperties: {
                                from: {
                                    val: '{"transform" : "s1 1", "opacity" : 1}',
                                    before: '{"transform" : "s0 0"}'
                                },
                                to: {
                                    val: '{"opacity" : 0}'
                                },
                                speed: 200
                            }
			        },
                        {
                            el: 'path:nth-child(1)',
                            animProperties: {
                                from: {
                                    val: '{"path" : "m 5.0916789,20.818994 53.8166421,0"}'
                                },
                                to: {
                                    val: '{"path" : "M 12.972944,50.936147 51.027056,12.882035"}'
                                }
                            }
			        },
                        {
                            el: 'path:nth-child(3)',
                            animProperties: {
                                from: {
                                    val: '{"path" : "m 5.0916788,42.95698 53.8166422,0"}'
                                },
                                to: {
                                    val: '{"path" : "M 12.972944,12.882035 51.027056,50.936147"}'
                                }
                            }
			        }
		        ]
                },
                options = {
                    easing: mina.elastic,
                    speed: 600,
                    evtoggle: 'click',
                    size: {
                        w: 64,
                        h: 64
                    },
                    toggle: false
                };

            function animateSVG(element) {
                var icon = Snap(element[0]);
                $log.debug(element, options.toggle);
                for (var i = 0, len = config.animation.length; i < len; ++i) {
                    $log.debug(i);
                    var a = config.animation[i],
                        el = icon.select(a.el),
                        animProp = options.toggle ? a.animProperties.from : a.animProperties.to,
                        val = animProp.val,
                        timeout = 0,
                        speed = a.animProperties.speed ? a.animProperties.speed : options.speed;
                    $log.debug(a, el, animProp, val, timeout, speed);
                    if (animProp.before) {
                        el.attr(JSON.parse(animProp.before));
                    }

                    setTimeout(function (el, val, animProp) {
                        return function () {
                            el.animate(JSON.parse(val), speed, options.easing, function () {
                                if (animProp.after) {
                                    icon.attr(JSON.parse(animProp.after));
                                }
                                if (animProp.animAfter) {
                                    icon.animate(JSON.parse(animProp.animAfter), speed, options.easing);
                                }
                            });
                        };
                    }(el, val, animProp), timeout * speed);

                }
                options.toggle = !options.toggle;
            }
            return function (scope, element, attr) {
                element.bind('click', function () {
                    animateSVG(element);
                });
            };
        }
    };
});