controllers.controller('FeedCtrl', ['$scope', '$state', 'datacontext', 'appConstants', '$ionicModal', '$ionicScrollDelegate', 'AnalyticsService', 'ObjectService', 'eventService', 'ActionService', 'PlatformCheck', '$filter', '$log', '$cordovaCamera', 'session', '$ionicLoading', 'appConfiguration', '$http', 'FeedService', '$translate', '$cordovaDialogs', 'PlatformCheck', 'NotifyService', function ($scope, $state, datacontext, appConstants, $ionicModal, $ionicScrollDelegate, AnalyticsService, ObjectService, eventService, ActionService, PlatformCheck, $filter, $log, $cordovaCamera, session, $ionicLoading, appConfiguration, $http, FeedService, $translate, $cordovaDialogs, PlatformCheck, NotifyService) {

    $scope.externalFeedPosts = [];
    $scope.hasMoreData = false;
    $scope.allowSendPosts = false;
    $scope.allowSendComments = false;
    $scope.allowSendLikes = false;
    var currentEvent;

    $scope.isDevice = PlatformCheck.isDevice();

    // Load More Logic
    var allPosts = [];
    var showUntil = 0;
    var loadInterval = 10;
    var isLoading = false;

    document.addEventListener("resume", onResume, false);
    function onResume() {
        setTimeout(function() {
            $ionicScrollDelegate.scrollTop();
            $scope.refreshContent(true);
            showUntil = 0;
            viewLoaded();
            }, 0);
    }

    $scope.$on("$ionicView.beforeEnter", function() {

        currentEvent = ObjectService.getSimpleObject(e.apiContentTypes.Event, datacontext.getCachedEventInformation(eventService.getId()));
        $scope.logoImageUrl = currentEvent.imageUrl;

        NotifyService.subscribe($scope, 'notify-eventExited', function eventExited() {
            $scope.externalFeedPosts = [];
            $scope.hasMoreData = false;
            $scope.allowSendPosts = false;
            $scope.allowSendComments = false;
            $scope.allowSendLikes = false;
            $scope.logoImageUrl = null;
            var currentEvent;
            $scope.twitterHashtag = null;
            $scope.currentUser = null;
            participantEntity = null;
            $scope.posts = null;

            // Load More Logic
            var allPosts = [];
            var showUntil = 0;
            var loadInterval = 10;
            var isLoading = false;
        });

    });

    $scope.$on('$ionicView.enter', function () {

        if ($scope.posts == undefined || $scope.posts == null) {
            $ionicScrollDelegate.scrollTop();
        }
        viewLoaded();

        
    });

    $scope.$on('$ionicView.leave', function (e) {
        var eventProperties = {
            "State Name": "Feed"
        };
        AnalyticsService.trackEvent("State Open", eventProperties);
    });

    function viewLoaded() {

        //$ionicScrollDelegate.scrollTop();
        AnalyticsService.startTimer("State Open");

        $scope.allowSendPosts = currentEvent.allowSendPosts;
        $scope.allowSendComments = currentEvent.allowSendComments;
        $scope.allowSendLikes = currentEvent.allowSendLikes;
        $scope.twitterHashtag = currentEvent.hashtags;
    
        if ($scope.posts == undefined || $scope.posts == null) {
            loadFeed();
            $scope.refreshContent(true);
        }
        

        participantEntity = datacontext.getCachedProfile(session.getLoggedInParticipantId());

        $scope.currentUser = ObjectService.getSimpleObject(e.apiContentTypes.Profile, participantEntity);


        //$scope.refreshContent();
        

        // $scope.refreshContent();


        //$scope.openModal();
        //$state.go("tabs.feedDetails");

    }

    function loadFeed() {

        allPosts = ObjectService.getSimpleObjectArray(e.apiContentTypes.ExternalPost, datacontext.getCachedFeedExternalContent())
        allPosts = $filter('filter')(allPosts, {
            isDeleted: '!true'
        });
        allPosts = $filter('orderBy')(allPosts, '-createdAt');

        showUntil = (allPosts.length > loadInterval) ? loadInterval : allPosts.length;
        $scope.posts = allPosts.slice(0, showUntil);
        $scope.hasMoreData = (allPosts.length > loadInterval) ? true : false;

        $ionicLoading.hide();
    }

    $scope.loadMore = function () {
        if (!isLoading && showUntil >= 10) {

            isLoading = true;
            var currentLength = showUntil;

            if ((showUntil + loadInterval) <= allPosts.length) {
                showUntil = showUntil + loadInterval;
                $scope.hasMoreData = true;
            } else {
                showUntil = allPosts.length;
                $scope.hasMoreData = false;
            }

            for (var i = currentLength; i <= showUntil; i++) {
                $scope.posts.push(allPosts[i]);
            }

            $scope.$broadcast('scroll.infiniteScrollComplete');
            isLoading = false;
        }

    }


    $scope.refreshContent = function (forced) {


        // Call sync to get updated content
        datacontext.sync(forced).then(function () {

            $log.debug("Sync Completed (feedController.js)");

            // Update feed from cache

            loadFeed();

            // Tell the refresher the refresh is complete
            $scope.$broadcast('scroll.refreshComplete');

        }).catch(function (err) {
            $log.debug("Fetch data failed. (app.js): " + err);
            $ionicLoading.hide(); 

        });
    }

    $scope.twitterButtonClicked = function () {

        var hashtag = currentEvent.hashtags;

        hashtag = hashtag.startsWith("#") ? hashtag.substr(1) : hashtag;

        // Open url in a new window
        var url = "https://www.twitter.com/hashtag/" + hashtag;

        if (url) {
            if (PlatformCheck.isDevice()) {
                window.open(url, '_blank', 'location=yes');
            } else {
                window.open(url, '_self', 'location=yes');
            }

        }
    }

    $scope.formatTime = function(date) {
      var now = moment.utc();
      var givenDate = moment(date.toString().replace(/"/g, ""));

      var difference = Math.floor((now - givenDate) / 86400000);
      if (difference <= 5) {
        // Less than 5 days old
        return moment(moment(givenDate).format('YYYY-MM-DD HH:mm:ss')).fromNow();
      } else {
        // More than 5 days old
        return givenDate.format("DD.MM.YYYY");
      }
    }

    $scope.likeButtonClicked = function (post) {
        var originalLikeState = post.isUserLiked;
        var originalLikeCount = post.likesCount;
        post.isUserLiked = !originalLikeState;

        if (originalLikeState === true) {
            // Post is already liked. Call Delete
            post.likesCount --;
            var likeRequest = FeedService.deleteLike(post.id);

            likeRequest.then(function () {
                post.isUserLiked = false;
                $scope.refreshContent(true);
            }, function () {
                post.isUserLiked = originalLikeState;
                post.likesCount = originalLikeCount;
                $translate.onReady().then(function () {
                    $cordovaDialogs.alert($translate.instant('NAVIGATION.FEED.MESSAGES.POST_LIKE_ERROR.BODY'), $translate.instant('NAVIGATION.FEED.MESSAGES.POST_LIKE_ERROR.HEADER'));
                });
            })
        } else {
            // Post is not liked. Call Post
            var likeRequest = FeedService.postLike(post.id);
            post.likesCount ++;
            likeRequest.then(function () {
                post.isUserLiked = true;
                $scope.refreshContent(true);
            }, function () {
                post.isUserLiked = originalLikeState;
                post.likesCount = originalLikeCount;
                $translate.onReady().then(function () {
                    $cordovaDialogs.alert($translate.instant('NAVIGATION.FEED.MESSAGES.POST_LIKE_ERROR.BODY'), $translate.instant('NAVIGATION.FEED.MESSAGES.POST_LIKE_ERROR.HEADER'));
                });
            })

        }
    }





    //////////////////
    // TICKET MODAL // 
    //////////////////

    $scope.postUpdate = [];
    $scope.postUpdate.comment = "";

    $ionicModal.fromTemplateUrl('post_update_modal.html', {
        scope: $scope,
        animation: 'slide-in-up'
    }).then(function (modal) {
        $scope.modal = modal;
        //modal.show();
    });

    $scope.openModal = function () {
        $scope.modal.show();
    };

    $scope.closeModal = function () {
        $scope.modal.hide();
    };

    // Cleanup the modal when we're done with it!
    $scope.$on('$destroy', function () {
        $scope.modal.remove();
    });

    // Execute action on hide modal
    $scope.$on('modal.hidden', function () {
        // Execute action
    });

    // Execute action on remove modal
    $scope.$on('modal.removed', function () {
        // Execute action
    });

    // Cancel Modal (delete content)
    $scope.cancelModal = function () {
        $scope.postUpdate.comment = '';
        $scope.postUpdate.photoUri = "";
        $scope.closeModal();

    }

    $scope.postButtonClicked = function () {
        $ionicLoading.show({
            template: '<ion-spinner></ion-spinner>'
        });

        if ($scope.postUpdate.photoUri != undefined && $scope.postUpdate.photoUri != "") {
            postWithImage();
        } else {
            postNoImage();
        }

    }

    function postWithImage(body) {

        // HTTP Call to post update
        var post = FeedService.postTextImage($scope.postUpdate.comment, $scope.postUpdate.photoUri);

        post.then(function () {
            $scope.refreshContent(true);
            
            $scope.cancelModal();
        }, function () {
            $translate.onReady().then(function () {
                $cordovaDialogs.alert($translate.instant('NAVIGATION.FEED.MESSAGES.POST_ERROR.BODY'), $translate.instant('NAVIGATION.FEED.MESSAGES.POST_ERROR.HEADER'));
            });
            $ionicLoading.hide();
        })
    }

    function postNoImage() {

        // HTTP Call to post update
        var post = FeedService.postText($scope.postUpdate.comment);

        post.success(function (data, status) {
            // $log.debug(JSON.stringify(data));
            $scope.refreshContent(true);
            $ionicLoading.hide();
            $scope.cancelModal();
        }).error(function () {
            $ionicLoading.hide();
            $translate.onReady().then(function () {
                $cordovaDialogs.alert($translate.instant('NAVIGATION.FEED.MESSAGES.POST_ERROR.BODY'), $translate.instant('NAVIGATION.FEED.MESSAGES.POST_ERROR.HEADER'));
            });
        });

    }

    $scope.cameraButtonClicked = function () {

        // Take Photo
        var options = {
            quality: 75,
            destinationType: destinationType(),
            sourceType: Camera.PictureSourceType.CAMERA,
            allowEdit: true,
            correctOrientation: true,

            encodingType: Camera.EncodingType.JPEG,
            popoverOptions: CameraPopoverOptions,
            targetWidth: 960,
            targetHeight: 960,
            saveToPhotoAlbum: false
        };
        $cordovaCamera.getPicture(options).then(function (imageUrl) {

            $scope.postUpdate.photoUri = imageUrl;
        }, function (error) {
            $log.debug(error);
        });
    }

    $scope.photosButtonClicked = function () {

        // upload from file system / gallery...
        var options = {
            quality: 75,
            destinationType: destinationType(),
            sourceType: Camera.PictureSourceType.PHOTOLIBRARY,
            allowEdit: true,
            encodingType: Camera.EncodingType.JPEG,
            popoverOptions: CameraPopoverOptions,
            targetWidth: 960,
            targetHeight: 960,
            saveToPhotoAlbum: false
        };

        $cordovaCamera.getPicture(options).then(function (imageUrl) {

            $scope.postUpdate.photoUri = imageUrl;


        }, function (error) {
            $log.debug(error);
        });
    }

    function destinationType() {
        var destinationType = null;
        if (ionic.Platform.isIOS()) {
            destinationType = Camera.DestinationType.FILE_URI;
        } else if (ionic.Platform.isAndroid()) {
            destinationType = Camera.DestinationType.FILE_URL;
        }

        return destinationType;
    }


}]).filter('getTitle', function() {
    return function(input) {
        var text = JSON.parse(input);
      return text.title;
    };
  }).filter('getMessage', function() {
    return function(input) {
        var text = JSON.parse(input);
      return text.message;
    };
  });