controllers.controller('FeedDetailsCtrl', ['$scope', '$state', 'datacontext', 'appConstants', 'smoothScroll', '$ionicScrollDelegate', 'FeedService', '$cordovaDialogs', '$translate', 'post', '$log', '$ionicLoading', '$ionicActionSheet', 'FlagContentService', 'session', '$ionicPopup', 'ObjectService', 'eventService', '$ionicHistory', function ($scope, $state, datacontext, appConstants, smoothScroll, $ionicScrollDelegate, FeedService, $cordovaDialogs, $translate, post, $log, $ionicLoading, $ionicActionSheet, FlagContentService, session, $ionicPopup, ObjectService, eventService, $ionicHistory) {

    $scope.post = post;
    $scope.comments = [];
    $scope.isLoadingComments = true;
    $scope.currentParticipantId = 0;
    $scope.allowSendPosts = false;
    $scope.allowSendComments = false;
    $scope.allowSendLikes = false;

    var currentEvent;

    $scope.$on("$ionicView.beforeEnter", function () {
        // If navigation history is empty (ie. browser refresh) --> reset state to parent tab
        if ($ionicHistory.viewHistory().currentView.backViewId == null) {
            $ionicHistory.nextViewOptions({
                disableAnimate: true,
                disableBack: true
            });
            $state.go("tabs.feed");
        }
    });

    $scope.$on('$ionicView.enter', function () {

        currentEvent = ObjectService.getSimpleObject(e.apiContentTypes.Event, datacontext.getCachedEventInformation(eventService.getId()));

        $scope.allowSendPosts = currentEvent.allowSendPosts;
        $scope.allowSendComments = currentEvent.allowSendComments;
        $scope.allowSendLikes = currentEvent.allowSendLikes;
        $scope.currentParticipantId = currentEvent.participantId;

        $scope.currentState = $state.current.name; 

        fetchComments();

    });

    $scope.refreshContent = function (forced) {

        // Call sync to get updated content
        datacontext.sync(forced).then(function () {

            $log.debug("Sync Completed (feedController.js)");

            // Update feed from cache

            $scope.post = datacontext.getCachedPost($scope.post.id);
            fetchComments();

        }).catch(function (err) {
            $log.debug("Fetch data failed. (app.js): " + err);

        });
    }

    function fetchComments() {
        var commentsRequest = FeedService.getComments($scope.post.id);
        commentsRequest.then(function (results) {

            $scope.isLoadingComments = false;
            $scope.comments = results.data;

            $ionicLoading.hide();
        }, function () {
            $scope.isLoadingComments = false;
            $ionicLoading.hide();
        });
    }

    // Triggered on a button click, or some other target
    $scope.openMenu = function () {

        $translate.onReady().then(function () {

            var destructiveButtonText = ($scope.currentParticipantId == $scope.post.participantId.id) ? $translate.instant('NAVIGATION.FEED_DETAILS.MENU.DELETE_POST') : "";

            // Show the action sheet
            var hideSheet = $ionicActionSheet.show({
                buttons: [{
                    text: $translate.instant('NAVIGATION.FEED_DETAILS.MENU.REPORT')
                }],
                cancelText: $translate.instant('NAVIGATION.FEED_DETAILS.MENU.CANCEL'),
                destructiveText: destructiveButtonText,
                cancel: function () {

                },
                buttonClicked: function (index) {
                    switch (index) {
                        case 0:
                            FlagContentService.flagContent(e.apiContentTypes.ExternalPost, $scope.post.id);
                            break;

                        default:
                            break;
                    }
                    return true;
                },
                destructiveButtonClicked: function () {
                    var confirmPopup = $ionicPopup.confirm({
                        title: $translate.instant('NAVIGATION.FEED_DETAILS.MENU.CONFIRM_DELETE_POST_POPUP.TITLE'),
                        template: $translate.instant('NAVIGATION.FEED_DETAILS.MENU.CONFIRM_DELETE_POST_POPUP.BODY'),
                        okText: $translate.instant('NAVIGATION.FEED_DETAILS.MENU.CONFIRM_DELETE_POST_POPUP.OK_BUTTON'),
                        okType: 'button-dark',
                        cancelText: $translate.instant('NAVIGATION.FEED_DETAILS.MENU.CONFIRM_DELETE_POST_POPUP.CANCEL_BUTTON'),
                        cancelType: ''
                    });

                    confirmPopup.then(function (res) {
                        if (res) {
                            // Continue

                            $ionicLoading.show({
                                template: '<ion-spinner></ion-spinner>'
                            });

                            var deleteRequest = FeedService.deletePost($scope.post.id);

                            deleteRequest.then(function () {
                                $state.go("tabs.feed");
                            }, function () {
                                $cordovaDialogs.alert($translate.instant('NAVIGATION.FEED_DETAILS.MENU.MESSAGES.DELETE_POST_ERROR.BODY'), $translate.instant('NAVIGATION.FEED_DETAILS.MENU.MESSAGES.DELETE_POST_ERROR.HEADER'));
                            })
                        }
                    })
                }
            });
        });
    };

    $scope.trackScroll = function () {
        //$ionicScrollDelegate.resize();
        var scrollDelegate = $ionicScrollDelegate.$getByHandle('agendaContent');
        var view = scrollDelegate.getScrollView();
        var top = "Top : " + $ionicScrollDelegate.getScrollPosition().top;
        // $log.debug("top:"+top);

    }

    $scope.likeButtonClicked = function (post) {
        var originalLikeState = post.isUserLiked;
        var originalLikeCount = post.likesCount;
        post.isUserLiked = !originalLikeState;

        if (originalLikeState === true) {
            // Post is already liked. Call Delete
            post.likesCount--;
            var likeRequest = FeedService.deleteLike(post.id);

            likeRequest.then(function () {
                post.isUserLiked = false;
                $scope.refreshContent(true);
            }, function () {
                post.isUserLiked = originalLikeState;
                post.likesCount = originalLikeCount;
                $translate.onReady().then(function () {
                    $cordovaDialogs.alert($translate.instant('NAVIGATION.FEED.MESSAGES.POST_LIKE_ERROR.BODY'), $translate.instant('NAVIGATION.FEED.MESSAGES.POST_LIKE_ERROR.HEADER'));
                });
            })
        } else {
            // Post is not liked. Call Post
            var likeRequest = FeedService.postLike(post.id);
            post.likesCount++;
            likeRequest.then(function () {
                post.isUserLiked = true;
                $scope.refreshContent(true);
            }, function () {
                post.isUserLiked = originalLikeState;
                post.likesCount = originalLikeCount;
                $translate.onReady().then(function () {
                    $cordovaDialogs.alert($translate.instant('NAVIGATION.FEED.MESSAGES.POST_LIKE_ERROR.BODY'), $translate.instant('NAVIGATION.FEED.MESSAGES.POST_LIKE_ERROR.HEADER'));
                });
            })

        }
    }

    $scope.postCommentButtonClicked = function () {

        if ($scope.commentText == undefined || $scope.commentText == "") {
            return;
        }

        $ionicLoading.show({
            template: '<ion-spinner></ion-spinner>'
        });

        var likeRequest = FeedService.postComment($scope.post.id, $scope.commentText);

        likeRequest.then(function () {
            $scope.refreshContent(true); // add refresh comments to refresh content function
            $scope.commentText = "";

        }, function () {
            $translate.onReady().then(function () {
                $cordovaDialogs.alert($translate.instant('NAVIGATION.FEED_DETAILS.MESSAGES.POST_COMMENT_ERROR.BODY'), $translate.instant('NAVIGATION.FEED_DETAILS.MESSAGES.POST_COMMENT_ERROR.HEADER'));
                $ionicLoading.hide();
            });
        })
    }

    $scope.deleteCommentClicked = function (commentId) {
        var confirmPopup = $ionicPopup.confirm({
            title: $translate.instant('NAVIGATION.FEED_DETAILS.CONFIRM_DELETE_COMMENT_POPUP.TITLE'),
            template: $translate.instant('NAVIGATION.FEED_DETAILS.CONFIRM_DELETE_COMMENT_POPUP.BODY'),
            okText: $translate.instant('NAVIGATION.FEED_DETAILS.CONFIRM_DELETE_COMMENT_POPUP.OK_BUTTON'),
            okType: 'button-dark',
            cancelText: $translate.instant('NAVIGATION.FEED_DETAILS.CONFIRM_DELETE_COMMENT_POPUP.CANCEL_BUTTON'),
            cancelType: ''
        });

        confirmPopup.then(function (res) {
            if (res) {
                // Continue

                $ionicLoading.show({
                    template: '<ion-spinner></ion-spinner>'
                });

                var deleteRequest = FeedService.deleteComment($scope.post.id, commentId);

                deleteRequest.then(function () {
                    $scope.refreshContent(true);
                }, function () {
                    $translate.onReady().then(function () {
                        // TODO: UPDATE TO LOCALIZED TEXT
                        $cordovaDialogs.alert($translate.instant('NAVIGATION.FEED_DETAILS.MESSAGES.DELETE_COMMENT_ERROR.BODY'), $translate.instant('NAVIGATION.FEED_DETAILS.MESSAGES.DELETE_COMMENT_ERROR.HEADER'));
                    });
                })
            }
        })
    }
}]);