/*
    LOCATION MAP CONTROLLER 
    BY: SIJAN GURUNG
    EVENTEYE AS
    NOV,2015
*/

controllers.controller('LocationMapCtrl', ['$scope', '$state', 'datacontext', '$log', '$ionicScrollDelegate', 'locationMap', '$timeout', 'NgMap', '$cordovaGeolocation', 'AnalyticsService', '$ionicHistory', function ($scope, $state, datacontext, $log, $ionicScrollDelegate, locationMap, $timeout, NgMap, $cordovaGeolocation, AnalyticsService, $ionicHistory) {

    $scope.locationMap = null;
    $scope.markers = [];
    $scope.currentLocation = [];

    document.addEventListener("deviceready", onDeviceReady, false);
    
    function onDeviceReady() {
        /**CORDOVA GEOLOCATION START*/
        var posOptions = {
            timeout: 10000,
            enableHighAccuracy: true
        };
        $cordovaGeolocation
            .getCurrentPosition(posOptions)
            .then(function (position) {
                $scope.currentLocation.latitude = position.coords.latitude
                $scope.currentLocation.longitude = position.coords.longitude
                $log.debug("lat:" + $scope.currentLocation.latitude + " ,long:" + $scope.currentLocation.longitude)
            }, function (err) {
                // error
                $log.debug("error:" + err.message);
            });
        /**CORDOVA GEOLOCATION ENDS*/
    } //End of device ready.....

    $scope.$on("$ionicView.beforeEnter", function() {
        // If navigation history is empty (ie. browser refresh) --> reset state to parent tab
        if ($ionicHistory.viewHistory().currentView.backViewId == null) {
            $ionicHistory.nextViewOptions({
                disableAnimate: true,
                disableBack: true
            });
            $state.go("tabs.menu");
        }       
    });

    $scope.$on('$ionicView.enter', function (e) {
        
        /*THIS PART WILL GET ALL THE MAPS WITH MARKERS AND ADD IT TO MARKERS VARIABLE....*/
        $scope.markers = []; //empty map... 
        $scope.locationMap = locationMap;

        // Analytics
        if (!ionic.Platform.isWebView()) {
            trackProfileView();
        } else {
            AnalyticsService.startTimer("Location Map State Open");
        }
        
        

        angular.forEach($scope.locationMap, function (locationMap) {
            $log.debug("Title:" + locationMap.title + " isDeleted for locationMap:" + locationMap.isDeleted);
            if (!locationMap.isDeleted) {
                angular.forEach(locationMap.markers, function (marker) {
                    $scope.markers.push(marker);
                });
            }
        });
        /*IF THE MARKERS VALUE IS EMPTY*/
        if (!$scope.locationMap.length) {
            $log.debug("no markers found!");

        } else
            $scope.checkMapAttributes(); //This is set the map to fit all the markers...

        $ionicScrollDelegate.resize()
        $scope.refreshContent();
        
        

    });
    
    $scope.$on('$ionicView.leave', function (e) {
        if (ionic.Platform.isWebView()) {
            trackProfileView();
        }
    });
    
    function trackProfileView() {
        var eventProperties = {
            "State Name": "Location Map",
            "Location Map Id": $scope.locationMap.id,
            "LocationMap Title": $scope.locationMap.title
        };
        AnalyticsService.trackEvent("Location Map State Open", eventProperties);
    }
    
    /*THIS METHOD WILL BE ON WHEN THE CUSTOM MARKER IS CLICKED
    @param event
    @param markerTitle
    */
    $scope.customMarkerClick = function (event, markerTitle) {
            $log.debug("Clicked Marker is :" + markerTitle);
            //alert(markerTitle);
            //this.style.display= 'none';
        } //end of customMArkerclick

    /*THIS METHOS WILL CHECK THE TOTAL MARKER WIDTH AND 
    SET THE MAP ZOOM PROPERTY ACCORDINGLY*/
    $scope.checkMapAttributes = function () {

            var positions = [];
            //fill position value....
            angular.forEach($scope.markers, function (marker) {
                positions.push([marker.latitude, marker.longitude]);
            });
            var bounds = new google.maps.LatLngBounds();
            for (var i = 0; i < positions.length; i++) {
                var latlng = new google.maps.LatLng(positions[i][0], positions[i][1]);
                bounds.extend(latlng);
            }

            var map;
            NgMap.getMap().then(function (evtMap) {
                map = evtMap;
                map.setCenter(bounds.getCenter());
                //map.fitBounds(bounds);
                map.setZoom(15);
            });
        } //end of checkMapAttributes...
    $scope.refreshContent = function (forced) {

        // Call sync to get updated content
        datacontext.sync(forced).then(function () {

            $log.debug("Sync Completed (locationMapController.js)");

            // Update list from cache
            $scope.locationMap = datacontext.getCachedLocationMap();

            // Tell the refresher the refresh is complete
            $scope.$broadcast('scroll.refreshComplete');

        }).catch(function (err) {
            $log.debug("Fetch data failed. (app.js): " + err);

        });
    }
}]);