controllers.controller('ParticipantsCtrl', ['$scope', 'datacontext', '$ionicFilterBar', '$filter', 'appConstants', '$log', 'AnalyticsService', '$ionicHistory', '$state', 'ObjectService', function ($scope, datacontext, $ionicFilterBar, $filter, appConstants, $log, AnalyticsService, $ionicHistory, $state, ObjectService) {
    // With the new view caching in Ionic, Controllers are only called
    // when they are recreated or on app start, instead of every page change.
    // To listen for when this page is active (for example, to refresh data),
    // listen for the $ionicView.enter event:
    //
    //$scope.$on('$ionicView.enter', function(e) {
    //});

    var filterBarInstance;

    var searchToFindLimit = 99999;
    var participants = [];
    $scope.participants = [];

    $scope.$on("$ionicView.beforeEnter", function () {
        // If navigation history is empty (ie. browser refresh) --> reset state to parent tab
        if ($ionicHistory.viewHistory().currentView.backViewId == null) {
            $ionicHistory.nextViewOptions({
                disableAnimate: true,
                disableBack: true
            });
            $state.go("tabs.menu");
        }
    });

    $scope.$on('$ionicView.enter', function (err) {

        // Analytics
        if (!ionic.Platform.isWebView()) {
            trackProfileView();
        } else {
            AnalyticsService.startTimer("Participant List State Open");
        }

        participants = ObjectService.getSimpleObjectArray(e.apiContentTypes.Participant, datacontext.getCachedParticipants());
        if (participants.length < searchToFindLimit) {
            $scope.participants = participants;
            $scope.searchToFind = false;
        } else {
            $scope.searchToFind = true;
        }
        $scope.refreshContent();
    });

    $scope.$on('$ionicView.leave', function () {

        if (ionic.Platform.isWebView()) {
            trackProfileView();
        }

    });

    function trackProfileView() {
        var eventProperties = {
            "State Name": "Participant List"
        };
        AnalyticsService.trackEvent("Participant List State Open", eventProperties);
    }

    $scope.gotoParticipantDetails = function (participantId) {
        // http://learn.ionicframework.com/formulas/sharing-data-between-views/
        // Get participant object by index, pass it to participantDetails

        $log.debug("Participant Clicked: " + participantId);
        var participantProfile = datacontext.getCachedParticipantProfile(participantId);
        $log.debug(participantProfile);
        $log.debug(participantProfile.firstName);
    };

    $scope.refreshContent = function (forced) {

        // Hide filterbar
        if (filterBarInstance) {
            filterBarInstance();
            filterBarInstance = null;
        }

        // Call sync to get updated content
        datacontext.sync(forced).then(function () {

            $log.debug("Sync Completed (syncController.js)");

            // Update list from cache
            participants = ObjectService.getSimpleObjectArray(e.apiContentTypes.Participant, datacontext.getCachedParticipants());
            if (participants.length < searchToFindLimit) {
                $scope.participants = participants;
                $scope.searchToFind = false;
            } else {
                $scope.searchToFind = true;
            }

            // Tell the refresher the refresh is complete
            $scope.$broadcast('scroll.refreshComplete');



        }).catch(function (err) {
            $log.debug("Fetch data failed. (app.js): " + err);

        });
    }

    $scope.showFilterBar = function () {
        filterBarInstance = $ionicFilterBar.show({
            items: participants,
            update: function (filteredItems) {
                $scope.participants = [];
                $scope.participants = filteredItems;
            },
            cancel: function () {
                $scope.search = "";
                if ($scope.searchToFind) {
                    $scope.participants = [];
                }
            },
            filter: function (collection, search) {
                $scope.search = search;
                if (!search)
                    return collection;
                var results = [];
                var searchString = search.toLowerCase();

                //    for (i = 0; i < collection.length; i++) {
                angular.forEach(collection, function (item) {
                    try {
                        var fName = item.firstName.toLowerCase();
                        var lName = item.lastName.toLowerCase();
                        var shouldAdd = false; //this would decide to add or not....
                        var searchCompany = appConstants.app.participantConfig.participantList.searchCompany;
                        var searchPosition = appConstants.app.participantConfig.participantList.searchPosition;

                        //Searching through firstname or lastname or both....
                        if ((fName.indexOf(searchString) !== -1) || (lName.indexOf(searchString) !== -1) || ((fName + " " + lName).indexOf(searchString) !== -1)) {
                            shouldAdd = true;
                        }
                        if ((searchPosition != false) && item.position != null) {
                            var position = item.position.toLowerCase();
                            //searching through company....
                            if (position.indexOf(searchString) !== -1) {
                                shouldAdd = true;
                            }
                        } //end of position search ...
                        if ((searchCompany != false) && item.companyName != null) {
                            var company = item.companyName.toLowerCase();
                            //searching through company....
                            if (company.indexOf(searchString) !== -1) {
                                shouldAdd = true;
                            }
                        } //end of position search ...

                        if (shouldAdd == true) {
                            results.push(item);
                        }
                    } catch (e) {

                    }


                });
                return results;
            }
        });
    };


}]);