controllers.controller('PollDetailCtrl', ['$scope', '$rootScope', '$state', 'datacontext', '$ionicScrollDelegate', 'PushService', '$stateParams', '$log', '$cordovaToast', 'PollService', 'pollData', '$ionicLoading', 'appConstants', '$cordovaDialogs', '$interval', 'AppService', '$translate', 'AnalyticsService', '$ionicHistory', function ($scope, $rootScope, $state, datacontext, $ionicScrollDelegate, PushService, $stateParams, $log, $cordovaToast, PollService, pollData, $ionicLoading, appConstants, $cordovaDialogs, $interval, AppService, $translate, AnalyticsService, $ionicHistory) {

    var refreshInterval = 3000;
    var timer;

    $scope.selected = [];
    $scope.noOfAnswers = 1; //this variable will set how many answers can a question have....
    $scope.firstTime = true; //This is to load a class for first time only for animation...

    
    $scope.myStyles = [];
    
    //by default make it true..so that it wont show the red bar onload....
    $scope.data={connection:true};

    $scope.$on("$ionicView.beforeEnter", function() {
        // If navigation history is empty (ie. browser refresh) --> reset state to parent tab
        if ($ionicHistory.viewHistory().currentView.backViewId == null) {
            $ionicHistory.nextViewOptions({
                disableAnimate: true,
                disableBack: true
            });
            $state.go("tabs.menu");
        }       
    });
    
    $scope.$on('$ionicView.enter', function (e) {

        $translate.onReady().then(function () {
            $ionicLoading.show({
                template: $translate.instant('NAVIGATION.POLL_DETAILS.LOADING')
            });
            refreshContent(pollData.data);
            $ionicScrollDelegate.resize();

            $ionicLoading.hide();

            startTimer();
            
        

            // Analytics
            if (!ionic.Platform.isWebView()) {
                trackProfileView();
            } else {
                AnalyticsService.startTimer("Poll Details State Open");
            }

            // FOR DEBUGGING ONLY:
            //$scope.pollData = pollData.data;

            //CHECKING FOR INTERNET CONNECTION...
            AppService.hasConnection().success(function(){
                //console.log("has connection");
                $scope.data={connection:true};
            }).error(function(err){
                //console.log("no connection");
                $scope.data={connection:false};
            });
        });

    });

    $scope.$on('$ionicView.leave', function () {
        if (angular.isDefined(timer)) {
            $interval.cancel(timer);
            timer = undefined;
        }
        
        if (ionic.Platform.isWebView()) {
            trackProfileView();
        }

    });
    
    function trackProfileView() {
        var eventProperties = {
            "State Name": "Poll Details",
            "Poll Id": $scope.pollData.id,
            "Poll Question": $scope.pollData.title
        };
        AnalyticsService.trackEvent("Poll Details State Open", eventProperties);
    }

    function startTimer() {
        // Start interval refresh
        timer = $interval(function () {
            $log.debug("Refreshing!");
            //Set the firstTime to be false ..
            $scope.firstTime = false; //this is to disable the animation..
            refreshContent();
        }, refreshInterval);
    }

    function refreshContent(newData) {

        if (newData) {
            angular.forEach(newData.alternatives, function (pollAnswer) {

                if (pollAnswer.hasVote === true)  {
                    $scope.showResults = newData.showResultsInApp;


                    $scope.complete = true;

                }

            });

            $scope.pollData = newData;
            setPercentage();

        } else {
            // HTTP Call to get active polls
            var updatedPollData = PollService.getPollDetails($stateParams.pollItemId);

            updatedPollData.success(function (data, status) {

                $log.debug("DATA: " + JSON.stringify(data));
                $scope.showResults = data.showResultsInApp;

                angular.forEach(data.alternatives, function (pollAnswer) {

                    if (pollAnswer.hasVote === true)  {

                        $scope.complete = true;

                    }

                });

                $scope.updatedPollData = data;
                //setPercentage();
                $scope.changeValue();

                $scope.data={connection:true};
            }).error(function(error){
                $scope.data={connection:false};
            });
        }
        //for both...
        //  $scope.changeValue();
    }

    //TESTING DYNAMIC VALUES...
    $scope.changeValue = function () {
            $scope.myStyles = [];
            //Now i am changing the value of percentage for all the answers...
            var totalVoteCount = $scope.updatedPollData.votesCount;
            angular.forEach($scope.updatedPollData.alternatives, function (pollAnswer) {
                var percentage = (pollAnswer.votesCount / totalVoteCount) * 100;
                if (pollAnswer.votesCount == undefined)
                    percentage = 0;
                pollAnswer.percentage = percentage;

                $log.debug("Changed Percentage for id:" + pollAnswer.id + " is " + pollAnswer.percentage);
                $scope.myStyles.push({
                    id: pollAnswer.id,
                    width: pollAnswer.percentage + '%'
                });
                //Now change the main $scope.pollData array....
                //THIS IS IMPORTANT ...... TO REFLECT THE CURRENT CHANGES...
                angular.forEach($scope.pollData.alternatives, function (oldPollAnswer) {
                    if (oldPollAnswer.id == pollAnswer.id) {
                        $log.debug("Old:" + oldPollAnswer.percentage + " changed to " + pollAnswer.percentage);
                        oldPollAnswer.votesCount = pollAnswer.votesCount;
                        oldPollAnswer.percentage = pollAnswer.percentage;
                    }
                });

            });

        } //end of testing changeValue....




    /* THIS METHOD WILL SET THE WIDTH FOR EACH ITEM */
    function setPercentage() {
        //as per the CSS the total width is 85%...
        var totalVoteCount = $scope.pollData.votesCount;
        angular.forEach($scope.pollData.alternatives, function (pollAnswer) {

            if (pollAnswer.votesCount == undefined || !pollAnswer.votesCount) {
                pollAnswer.votesCount = 0;
            }

            //Now calculate according to percentage...
            var percentage = (pollAnswer.votesCount / totalVoteCount) * 100;
            //If the percentage is above 100.. set it to 100%
            if (percentage > 100)
                percentage = 100;
            pollAnswer.percentage = percentage; //THIS WILL ADD A ELEMENT INTO THE MAIN ARRAY 
            $scope.myStyles.push({
                id: pollAnswer.id,
                width: pollAnswer.percentage + '%'
            });

        });
    } /* end of setPercentage*/


    $scope.getPercentage = function (id) {
        var testing;
        angular.forEach($scope.myStyles, function (myStyle) {
            //$log.debug("recId:"+id+"id:"+myStyle.id+" width: "+myStyle.width);
            if (id == myStyle.id) {
                //      $log.debug("matched with :"+myStyle.width);
                testing = myStyle.width;
            }
        });
        return {
            width: testing
        }

    }


    $scope.alternativeClicked = function (alternativeId) {

        $translate.onReady().then(function () {

            $ionicLoading.show({
                template: $translate.instant('NAVIGATION.POLL_DETAILS.MESSAGES.VOTING')
            });

        });


        var pollPromise = PollService.putVote($scope.pollData.id, alternativeId);
        pollPromise.success(function (data, status) {

            /*

                        var seen = []
                        var json = JSON.stringify($scope.pollData, function (key, val) {
                            if (typeof val == "object") {
                                if (seen.indexOf(val) >= 0)
                                    return
                                seen.push(val)
                            }
                            return val
                        }, 0)



                        $log.debug("Response from vote(" + status + "): " + json);
                        */
            refreshContent(data);
            if ($scope.complete) {
                startTimer();
            }
            $ionicLoading.hide();

        }).error(function () {
            $translate.onReady().then(function () {
                $cordovaDialogs.alert($translate.instant('NAVIGATION.POLL_DETAILS.MESSAGES.VOTING_ERROR.BODY'), $translate.instant('NAVIGATION.POLL_DETAILS.MESSAGES.VOTING_ERROR.TITLE'));
                $ionicLoading.hide();
            });

        });
    }



}])

.directive("progressbar", function () {
    return {
        restrict: "EA",
        scope: {
            widthValue: "="
        },
        link: function (scope, element, attrs) {

            scope.$watch("widthValue", function (value) {
                element.css("width", scope.widthValue + "%");
            });
        }
    };
});