controllers.controller('SigninCtrl', ['$scope', '$state', '$timeout', 'session', 'appConstants', 'appConfiguration', '$ionicHistory', '$ionicLoading', '$cordovaDialogs', '$window', '$log', 'PlatformCheck', '$translate', 'datacontext', 'eventService', 'ObjectService', 'AnalyticsService', '$ionicPopup', '$filter', function ($scope, $state, $timeout, session, appConstants, appConfiguration, $ionicHistory, $ionicLoading, $cordovaDialogs, $window, $log, PlatformCheck, $translate, datacontext, eventService, ObjectService, AnalyticsService, $ionicPopup, $filter) {

    $scope.loginButtonsDisabled = false;
    $scope.credentials = {
        email: "",
        appId: appConstants.app.appId
    };

    // Check if user has already entered email
    $scope.$on('$ionicView.beforeEnter', function () {
        var selectedEmail = $window.localStorage.getItem(appConfiguration.localStorageKeys.auth.lastUsedEmailKey);
        if (selectedEmail != undefined && selectedEmail && selectedEmail.length > 0) {
            $ionicHistory.nextViewOptions({
                disableAnimate: true
            });
            $state.go("auth.signinVerification");
        }
    });

    $scope.$on('$ionicView.enter', function () {

        // Analytics
        if (!ionic.Platform.isWebView()) {
            trackProfileView();
        } else {
            AnalyticsService.startTimer("Login State Open");
        }
    });

    function trackProfileView() {
        var eventProperties = {
            "Login State Open": "Email Entry State"
        };
        AnalyticsService.trackEvent("Login State Open", eventProperties);
    }

    $scope.$on('$ionicView.leave', function (e) {
        if (ionic.Platform.isWebView()) {
            trackProfileView();
        }
    });

    $scope.submitEmail = function () {
        $translate.onReady().then(function () {
            // Check to make sure email input is not empty
            if (!$scope.credentials.email) {

                $ionicLoading.hide();
                $cordovaDialogs.alert($translate.instant('AUTH.LOGIN_EMAIL.MESSAGES.VALIDATION_ERROR.BODY'), $translate.instant('AUTH.LOGIN_EMAIL.MESSAGES.VALIDATION_ERROR.TITLE'));

                $scope.loginButtonsDisabled = false;
                return;
            }
            var confirmPopup = $ionicPopup.confirm({
                title: $translate.instant('AUTH.LOGIN_EMAIL.DIALOGS.TERMS.TITLE'),
                cssClass: 'exitConfirm',
                template: $translate.instant('AUTH.LOGIN_EMAIL.DIALOGS.TERMS.BODY'),
                okText: $translate.instant('AUTH.LOGIN_EMAIL.DIALOGS.TERMS.OK_BUTTON'),
                okType: 'button-calm',
                cancelText: $translate.instant('AUTH.LOGIN_EMAIL.DIALOGS.TERMS.CANCEL_BUTTON'),
                cancelType: 'button-assertive'
            });
            confirmPopup.then(function (res) {
                if (res) {
                    $scope.loginButtonsDisabled = true;

                    $ionicLoading.show({
                        template: '<ion-spinner></ion-spinner>'
                    });                    

                    var verification = session.requestToken($scope.credentials);
                    verification.success(function (data, status) {
                        $log.debug("Verification (" + status + "): Email Sent: " + $scope.credentials.email);
                        $window.localStorage.setItem(appConfiguration.localStorageKeys.auth.lastUsedEmailKey, $scope.credentials.email);
                        $log.debug("Email locally set: " + $window.localStorage.getItem(appConfiguration.localStorageKeys.auth.lastUsedEmailKey));
                        $ionicLoading.hide();

                        // If logging in as demo user
                        //if ($scope.credentials.email == appConstants.auth.demoLoginEmail) {

                            if (PlatformCheck.isDevice()) {
                                $scope.loginButtonsDisabled = false;
                                $state.go("auth.enablePushNotifications");
                            } else {
                                $scope.loginButtonsDisabled = false;
                                $state.go("multi.eventList");
                            }


                        //} else {
                        //    $scope.loginButtonsDisabled = false;
                        //    $state.go("auth.signinVerification");
                        //}

                    }).error(function (response) {
                        //$scope.errorMessage = response.message;


                        $scope.loginButtonsDisabled = false;
                        $ionicLoading.hide();
                        $cordovaDialogs.alert($translate.instant('AUTH.LOGIN_EMAIL.MESSAGES.GENERAL_ERROR.BODY'), $translate.instant('AUTH.LOGIN_EMAIL.MESSAGES.GENERAL_ERROR.TITLE'));
                        $log.debug("Error: " + response);

                    });

                    //return false;

                    /*
                    console.log("1 page EMAIL:" + $scope.credentials.email);
                    $window.localStorage.setItem(appConfiguration.localStorageKeys.auth.lastUsedEmailKey, $scope.credentials.email);
            
                    $state.go("auth.signinVerification");
                    */
                } else {
                    // Terms not accepted
                }
            });

        });






    }

    $scope.test = function () {
        console.log("tapped!");
    }

    $scope.clearHistory = function () {
        $ionicHistory.clearHistory();
    }

    $scope.showURL = function (which) {
        if (which == 1) {
            window.open(appConfiguration.privacyPolicyUrl, '_blank');
        } else if(which ==2)
            $cordovaDialogs.alert("If you need help with the sign-in please e-mail your question to TechCampus@egencia.com.", "Support");
            /* window.open(appConstants.app.helpDeskUrl, '_blank'); */
            return false;
        };

}]);