controllers.controller('SpeakersCtrl', ['$scope', 'datacontext', '$ionicFilterBar', '$filter', 'appConstants', '$log', 'ObjectService', 'AnalyticsService', '$ionicHistory', '$state', function ($scope, datacontext, $ionicFilterBar, $filter, appConstants, $log, ObjectService, AnalyticsService, $ionicHistory, $state) {

    var filterBarInstance;

    if (!$scope.speakers || $scope.speakers == undefined) {
        $scope.speakers = [];
    }

    $scope.showPosition = appConstants.app.speakerConfig.speakerList.position;
    $scope.showCompany = appConstants.app.speakerConfig.speakerList.company;

    $scope.$on("$ionicView.beforeEnter", function() {
        // If navigation history is empty (ie. browser refresh) --> reset state to parent tab
        if ($ionicHistory.viewHistory().currentView.backViewId == null) {
            $ionicHistory.nextViewOptions({
                disableAnimate: true,
                disableBack: true
            });
            $state.go("tabs.menu");
        }       
    });

    $scope.$on('$ionicView.enter', function () {
        
        // Analytics
        if (!ionic.Platform.isWebView()) {
            trackProfileView();
        } else {
            AnalyticsService.startTimer("Speaker List State Open");
        }
        
        var speakers = datacontext.getCachedSpeakers();
        $scope.speakers = ObjectService.getSimpleObjectArray(e.apiContentTypes.Speaker, speakers);
        $scope.refreshContent();
    });
    
    $scope.$on('$ionicView.leave', function (e) {
        if (ionic.Platform.isWebView()) {
            trackProfileView();
        }
    });
    
    function trackProfileView() {
        var eventProperties = {
            "State Name": "Speaker List"
        };
        AnalyticsService.trackEvent("Speaker List State Open", eventProperties);
    }

    /* THIS WILL SET THE HEIGHT OF EACH ITEM...*/
    $scope.setItemHeight = function (speakerItem) {
        //if the sub-heading is too long...set size big...
        var totalChars = 0;
        if (speakerItem.position != null) {
            totalChars += speakerItem.position.length;
        }
        if (speakerItem.companyName != null) {
            totalChars += speakerItem.companyName.length;
        }
        if (totalChars >= 50) //if subheader is more than 50 chars....then bigItem height....
            return '92px';
        else
            return '75px';
        return '10px'; //This will be used for sectionHeader.....
    }
    $scope.refreshContent = function (forced) {

        // Hide filterbar
        if (filterBarInstance) {
            filterBarInstance();
            filterBarInstance = null;
        }

        // Call sync to get updated content
        datacontext.sync(forced).then(function () {

            // Update list from cache
            var speakers = datacontext.getCachedSpeakers();
            $scope.speakers = ObjectService.getSimpleObjectArray(e.apiContentTypes.Speaker, speakers);

            // Tell the refresher the refresh is complete
            $scope.$broadcast('scroll.refreshComplete');

        }).catch(function (err) {
            $log.debug("Fetch data failed. (app.js): " + err);

        });
    }

    $scope.showFilterBar = function () {
        filterBarInstance = $ionicFilterBar.show({
            items: $scope.speakers,
            update: function (filteredItems) {
                $scope.speakers = filteredItems;
            },
            filter: function (collection, search) {
                if (!search)
                    return collection;
                var results = [];
                var searchString = search.toLowerCase();

                //    for (i = 0; i < collection.length; i++) {
                angular.forEach(collection, function (item) {
                    var fName = item.firstName.toLowerCase();
                    var lName = item.lastName.toLowerCase();
                    var shouldAdd = false; //this would decide to add or not....
                    var searchCompany = appConstants.app.speakerConfig.speakerList.searchCompany;
                    var searchPosition = appConstants.app.speakerConfig.speakerList.searchPosition;

                    //Searching through firstname or lastname or both....
                    if ((fName.indexOf(searchString) !== -1) || (lName.indexOf(searchString) !== -1) || ((fName + " " + lName).indexOf(searchString) !== -1)) {
                        shouldAdd = true;
                    }
                    if ((searchPosition != false) && item.position != null) {
                        var position = item.position.toLowerCase();
                        //searching through company....
                        if (position.indexOf(searchString) !== -1) {
                            shouldAdd = true;
                        }
                    } //end of position search ...
                    if ((searchCompany != false) && item.companyName != null) {
                        var company = item.companyName.toLowerCase();
                        //searching through company....
                        if (company.indexOf(searchString) !== -1) {
                            shouldAdd = true;
                        }
                    } //end of position search ...

                    if (shouldAdd == true) {
                        results.push(item);
                    }

                });
                return results;
            }
        });
    };


            }]);