controllers.controller('SyncCtrl', ['$scope', '$state', 'datacontext', 'appConstants', '$timeout', '$ionicHistory', 'PushService', '$log', 'PlatformCheck', '$window', 'appConfiguration', 'session', '$cordovaNetwork', 'eventService', '$translate', 'ObjectService', 'AppService', 'CustomColorService', '$ionicModal', '$stateParams', function ($scope, $state, datacontext, appConstants, $timeout, $ionicHistory, PushService, $log, PlatformCheck, $window, appConfiguration, session, $cordovaNetwork, eventService, $translate, ObjectService, AppService, CustomColorService, $ionicModal, $stateParams) {

    $scope.isSyncing = true;

    $translate.onReady().then(function () {
        $scope.syncMessageTitle = $translate.instant('AUTH.SYNC.IN_PROGRESS.TITLE');
        $scope.syncMessage = $translate.instant('AUTH.SYNC.IN_PROGRESS.BODY');
    });

    $scope.startSync = function () {

        $scope.isSyncing = true;
        $translate.onReady().then(function () {
            $scope.syncMessageTitle = $translate.instant('AUTH.SYNC.IN_PROGRESS.TITLE');
            $scope.syncMessage = $translate.instant('AUTH.SYNC.IN_PROGRESS.BODY');
        });


        var connectionPromise = AppService.hasConnection();
        connectionPromise.success(function (data, status) {
            // Has connection to server
            $log.debug("Connection Check: " + data + " " + status);

            datacontext.sync(true).then(function () {

                $log.debug("Sync Completed (syncController.js)");

                var currentEvent = ObjectService.getSimpleObject(e.apiContentTypes.Event, datacontext.getCachedEventInformation(eventService.getId()));

                var participantEntity = datacontext.getCachedProfile(session.getLoggedInParticipantId());

                var participant = ObjectService.getSimpleObject(e.apiContentTypes.Profile, participantEntity);

                CustomColorService.updateColorScheme();

                $translate.use(currentEvent.language);
                moment.locale(currentEvent.language.substring(0, 2));

                var template = '<div class="modal image-modal blackBackground"><ion-pane class="blackBackground"><img img-cache ic-src="' + currentEvent.eventSplashScreenImageUrl + '" class="fullscreen-image"/></ion-pane></div>'
                //$rootScope.modal = $ionicModal.fromTemplate('<div class="modal image-modal blackBackground"><ion-pane class="blackBackground"><img ng-src="../img/demo_splash.png" class="fullscreen-image"/></ion-pane></div>', {
                $scope.modal = $ionicModal.fromTemplate(template, {
                    scope: $scope,
                    animation: 'none'
                });

                $scope.openModal = function () {
                    if (appConstants.app.isContainerApp) {
                        $scope.modal.show();
                    }
                };

                $scope.closeModal = function () {
                    $scope.modal.hide();
                };

                //Cleanup the modal when we're done with it!
                $scope.$on('$destroy', function () {
                    $scope.modal.remove();
                });
                // Execute action on hide modal
                $scope.$on('modal.hide', function () {
                    // Execute action
                });
                // Execute action on remove modal
                $scope.$on('modal.removed', function () {
                    // Execute action
                });
                $scope.$on('modal.shown', function () {
                    //console.log('Modal is shown!');
                });

                //console.log("logged text: " + currentEvent.eventSplashScreenImageUrl);
                //console.log(currentEvent);
                if (currentEvent.eventSplashScreenImageUrl != "") {
                    $scope.openModal();

                    $timeout(function () {
                        //console.log("called");
                        $scope.closeModal();
                    }, appConstants.app.eventSplashScreenDelay);
                }


                try {
                    var dateLastUpdated = moment(participant.updatedAt);
                    var dateToday = moment();

                    // Check if Profile Module is set to show
                    if (currentEvent.moduleProfile) {
                        // Check if profile is empty

                        // Check if difference between today and last profile update is greater than internval for notification specified in appConfiguration
                        if (dateToday.diff(dateLastUpdated, 'days') > appConstants.app.participantConfig.editProfile.updateProfileReminderDays) {
                            // Difference is greater than the limit set in appConfiguration
                            $state.go("multi.updateProfile");
                        } else if ($stateParams.forceProfileUpdate) {
                            $state.go("multi.updateProfile", {newProfile: true});
                        } else {
                            if (participant.imageUrl == "" &&  participant.mobile == "" && participant.twitter == "" &&  participant.description == "" && participant.linkedIn == "") {
                                $state.go("multi.updateProfile");

                            } else {
                                $state.go(appConstants.app.defaultAuthorizedState);
                            }
                        }
                    } else {
                        $state.go(appConstants.app.defaultAuthorizedState);
                    }



                } catch (e) {
                    $state.go(appConstants.app.defaultAuthorizedState);
                }

            }, function (error) {
                $log.debug("Fetch data failed. (app.js)");

                var stashName = appConfiguration.localStorageKeys.stashName(eventService.getId());

                //window.localStorage.setItem(stashName, null);
                var importData = window.localStorage.getItem(stashName);
                $log.debug("DATA CHECK CATCH: " + stashName + "/////////" + importData);
                if (typeof importData !== 'undefined' && importData !== null) {
                    datacontext.loadData();
                    $state.go(appConstants.app.defaultAuthorizedState);
                } else {
                    showError();
                }

            });

        }).error(function (data, status) {
            // No connection to server
            $log.debug("Connection Check: " + data + " " + status);
            var stashName = appConfiguration.localStorageKeys.stashName(eventService.getId());
            var importData = window.localStorage.getItem(stashName);
            $log.debug("Stash: " + stashName + "/////////" + importData);
            if (typeof importData !== 'undefined' && importData !== null) {
                datacontext.loadData();
                $state.go(appConstants.app.defaultAuthorizedState);
            } else {
                showError();
            }

        });

    }

    function showError() {
        eventService.destroy();
        datacontext.loadData();
        $scope.isSyncing = false;
        $translate.onReady().then(function () {
            $scope.syncMessageTitle = $translate.instant('AUTH.SYNC.ERROR.TITLE');
            $scope.syncMessage = $translate.instant('AUTH.SYNC.ERROR.BODY');
        });
    }

    $scope.$on('$ionicView.enter', function (e) {

        if ($window.localStorage.getItem(appConfiguration.localStorageKeys.stashName(session.getCurrentEventId)) === null) {
            $log.debug("Stash doesn't exist. Syncing...");
            $scope.startSync();
        } else {
            $log.debug("Stash Already Exists. Moving on...");
            $state.go(appConstants.defaultAuthorizedState);
        }



    });

    $scope.clearHistory = function () {
        $ionicHistory.clearHistory();
    }

    $scope.enablePushNotifications = function () {
        $log.debug("Calling registerdevice");
        PushService.registerDevice();
        enterApp();
        //$state.go("multi.eventList");
    }

    $scope.noAction = function () {
        enterApp();
        //$state.go("multi.eventList");
    }

    function enterApp() {
        // If one event available AND this event should be autoloading --> Open Event. Else --> go to multi-list

        datacontext.getEvents().then(function () {
            var currentEvents = ObjectService.getSimpleObjectArray(e.apiContentTypes.Event, datacontext.getCachedEvents());
            $log.debug("Current Events: " + JSON.stringify(currentEvents));
            $log.debug("New Count: " + currentEvents.length + ". Object: " + JSON.stringify(currentEvents) + ". Autoload: " + currentEvents[0].autoloadEvent);
            if (currentEvents.length == 1 && currentEvents[0] && currentEvents[0].autoloadEvent) {
                $log.debug("app.js: 1 event, data available, autoload true");
                datacontext.clearData();

                var token = session.getToken();
                var userParticipantId = session.getLoggedInParticipantId();
                var eventToken = currentEvents[0].authToken;
                var currentEventId = currentEvents[0].id;

                session.create('', token, true, userParticipantId, eventToken);

                eventService.create(currentEventId);

                $state.go("auth.sync");
            } else {
                $log.debug("app.js: conditions false, loading multi list");
                $state.go("multi.eventList");
            }
        });
    }


}]);