controllers.controller('VerificationCtrl', ['$scope', '$state', '$timeout', 'session', 'appConstants', 'appConfiguration', '$ionicHistory', '$ionicLoading', '$cordovaDialogs', '$window', '$log', 'PlatformCheck', '$translate', 'datacontext', 'eventService', 'ObjectService', '$ionicActionSheet', 'AnalyticsService', function ($scope, $state, $timeout, session, appConstants, appConfiguration, $ionicHistory, $ionicLoading, $cordovaDialogs, $window, $log, PlatformCheck, $translate, datacontext, eventService, ObjectService, $ionicActionSheet, AnalyticsService) {
    
	$scope.loginButtonsDisabled = false;

    $scope.credentials = {
        email: "",
        pin: "",
		appId: appConstants.app.appId
    };
	
	//console.log("SELECTED EMAIL:" + $scope.selectedEmail);

    // Check if user has already entered email
	$scope.$on('$ionicView.beforeEnter', function () {
		var selectedEmail = $window.localStorage.getItem(appConfiguration.localStorageKeys.auth.lastUsedEmailKey);
		if (selectedEmail == undefined || !selectedEmail || selectedEmail == "") {
			$ionicHistory.nextViewOptions({
				disableAnimate: true
			});
			$state.go("auth.login");
		}
	});

    $scope.$on('$ionicView.enter', function (e) {
		$scope.selectedEmail = $window.localStorage.getItem(appConfiguration.localStorageKeys.auth.lastUsedEmailKey);
        
        // Analytics
        if (!ionic.Platform.isWebView()) {
            trackProfileView();
        } else {
            AnalyticsService.startTimer("Login State Open");
        }
    });

    function trackProfileView() {
        var eventProperties = {
            "Login State Open": "PIN Entry State"
        };
        AnalyticsService.trackEvent("Login State Open", eventProperties);
    }

    $scope.$on('$ionicView.leave', function (e) {
        if (ionic.Platform.isWebView()) {
            trackProfileView();
        }
    });

    $scope.verifyPIN = function () {

        $scope.loginButtonsDisabled = true;
        $ionicLoading.show({
            template: '<ion-spinner></ion-spinner>'
        });

        var pin = $scope.credentials.pin;
        var email = $scope.selectedEmail;

        if (!$scope.credentials.pin || $scope.credentials.pin == '' || !$scope.selectedEmail || $scope.selectedEmail == '') {

            $cordovaDialogs.alert($translate.instant('AUTH.SIGNIN_VERIFICATION.MESSAGES.VALIDATION_ERROR.BODY'), $translate.instant('AUTH.SIGNIN_VERIFICATION.MESSAGES.VALIDATION_ERROR.TITLE'));
            $scope.loginButtonsDisabled = false;
            $ionicLoading.hide();

            return;
        }

        var credentials = {
            pin: pin,
            email: $scope.selectedEmail,
            appId: appConstants.app.appId
        }

        $log.debug("Logging in with the following credentials: " + credentials);

        var authentication = session.verifyPIN(credentials);
        authentication.success(function (data, status) {
            $timeout(function () {
                $log.debug("Verification (" + status + "): Verified!");
                $ionicLoading.hide();

                if (PlatformCheck.isDevice()) {
                    $scope.loginButtonsDisabled = false;
                    $state.go("auth.enablePushNotifications");
                } else {
                    $scope.loginButtonsDisabled = false;
                    enterApp();
                }
            }, appConstants.app.debugging.standardTimeoutDelay);
        }).error(function (response) {
            // TODO: Add error handling
            $log.debug("Hiding $ionicLoading");
            $ionicLoading.hide();
            $scope.loginButtonsDisabled = false;
            $cordovaDialogs.alert($translate.instant('AUTH.SIGNIN_VERIFICATION.MESSAGES.VALIDATION_ERROR.BODY'), $translate.instant('AUTH.SIGNIN_VERIFICATION.MESSAGES.VALIDATION_ERROR.TITLE'));
            
        });

    };

    function enterApp() {
        // If one event available AND this event should be autoloading --> Open Event. Else --> go to multi-list
        try {
            datacontext.getEvents().then(function () {
                var currentEvents = ObjectService.getSimpleObjectArray(e.apiContentTypes.Event, datacontext.getCachedEvents());
                $log.debug("Count: " + currentEvents.length + ". Object: " + JSON.stringify(currentEvents[0])); //+ ". Autoload: " + currentEvents[0].autoloadEvent);
                if (currentEvents.length == 1 && currentEvents[0] && currentEvents[0].autoloadEvent) {
                    $log.debug("app.js: 1 event, data available, autoload true");
                    datacontext.clearData();

                    var token = session.getToken();
                    var userParticipantId = session.getLoggedInParticipantId();
                    var eventToken = currentEvents[0].authToken;
                    var currentEventId = currentEvents[0].id;

                    session.create('', token, true, userParticipantId, eventToken);

                    eventService.create(currentEventId);

                    $state.go("auth.sync");
                } else {
                    $log.debug("app.js: conditions false, loading multi list");
                    $state.go("multi.eventList");
                }
            });
        } catch (e) {
            $state.go("multi.eventList");
        }


    }

	 // Triggered on a button click, or some other target
 	$scope.showHelpMenu = function() {
		$translate.onReady().then(function () {
                    
			// Show the action sheet
			var hideSheet = $ionicActionSheet.show({
				buttons: [
				{ text: $translate.instant('AUTH.SIGNIN_VERIFICATION.SUPPORT.OPTIONS.FAQ') },
				{ text: $translate.instant('AUTH.SIGNIN_VERIFICATION.SUPPORT.OPTIONS.CONTACT_SUPPORT') }
				],
				titleText: $translate.instant('AUTH.SIGNIN_VERIFICATION.SUPPORT.ACTION_MENU_TITLE'),
				cancelText: $translate.instant('AUTH.SIGNIN_VERIFICATION.SUPPORT.OPTIONS.CANCEL'),
				cancel: function() {
					// add cancel code..
					},
				buttonClicked: function(index) {
					switch(index) {
						case 0:
						{
							// FAQ: 
							window.open('https://eventeye.freshdesk.com/support/solutions/folders/35000091301', '_blank');
							break;
						}
						case 1: 
						{
							// Contact Support
							window.open(appConstants.app.helpDeskUrl, '_blank');
							break;
						}
					}
				return true;
				}
			});
		});

	};

    $scope.resetLogin = function ()  {
        $window.localStorage.removeItem(appConfiguration.localStorageKeys.auth.lastUsedEmailKey);
        $state.go("auth.login");
    }

    $scope.clearHistory = function () {
        $ionicHistory.clearHistory();
    }

}]);