app.factory('model', function () {
        
    var DT = breeze.DataType; // alias
    return {
        initialize: initialize
    }
    
    function initialize(metadataStore) {
       
        /*
         * Participant Model
         */
        
        metadataStore.addEntityType({
            shortName: "Participant",
            namespace: "EventEye.Api.Models.Contexts",
            autoGeneratedKeyType: breeze.AutoGeneratedKeyType.Identity,
            dataProperties: {
                id:             { dataType: DT.Int64, isPartOfKey: true },
                is:             { dataType: DT.Int64 },
                
                firstName:      { dataType: DT.String },
                lastName:       { dataType: DT.String },
                email:          { dataType: DT.String },
                tagLine:        { dataType: DT.String },
                description:    { dataType: DT.String },
                imageUrl:       { dataType: DT.String },
                mobile:         { dataType: DT.String },
                twitter:        { dataType: DT.String },
                linkedIn:       { dataType: DT.String },
                companyName:    { dataType: DT.String },
                position:       { dataType: DT.String },
                customField:    { dataType: DT.String },

                //postIds:        { dataType: DT.Undefined },
                
                isPublicEmail:  { dataType: DT.Boolean },
                isPublic:       { dataType: DT.Boolean },
                isDeleted:      { dataType: DT.Boolean },
                
                updatedAt:      { dataType: DT.DateTime}
            }
            
        });
        
        
        /*
         * Profile Model
         */
        
        //TO BE COMPLETED
        
        metadataStore.addEntityType({
            shortName: "Profile",
            namespace: "EventEye.Api.Models.Contexts",
            autoGeneratedKeyType: breeze.AutoGeneratedKeyType.Identity,
            dataProperties: {
                id:                 { dataType: DT.Int64, isPartOfKey: true },
                is:                 { dataType: DT.Int64 },
                
                firstName:          { dataType: DT.String },
                lastName:           { dataType: DT.String },
                email:              { dataType: DT.String },
                tagLine:            { dataType: DT.String },
                description:        { dataType: DT.String },
                imageUrl:           { dataType: DT.String },
                mobile:             { dataType: DT.String },
                twitter:            { dataType: DT.String },
                linkedIn:           { dataType: DT.String },
                companyName:        { dataType: DT.String },
                position:           { dataType: DT.String },
                customField:        { dataType: DT.String },
                checkInCode:        { dataType: DT.String },
                programLoginCode:   { dataType: DT.String },
                
                isPublicEmail:      { dataType: DT.Boolean },
                isPublic:           { dataType: DT.Boolean },
                isDeleted:          { dataType: DT.Boolean },
                
                updatedAt:          { dataType: DT.DateTime}
            }
            
        });
        
        
        
        
        /*
         * ScheduleItem Model
         */
        
        metadataStore.addEntityType({
            shortName: "ScheduleItem",
            namespace: "EventEye.Api.Models.Contexts",
            autoGeneratedKeyType: breeze.AutoGeneratedKeyType.Identity,
            dataProperties: {
                id:                 { dataType: DT.Int64, isPartOfKey: true },
                is:                 { dataType: DT.Int64 },
                
                title:              { dataType: DT.String },
                subTitle:           { dataType: DT.String },              
                description:        { dataType: DT.String },
                imageUrl:           { dataType: DT.String },
                hashtags:           { dataType: DT.String },
                actions:            { dataType: DT.String },
                
                startTime:          { dataType: DT.DateTime },     
                endTime:            { dataType: DT.DateTime },

                featured:           { dataType: DT.Boolean },
                isAttend:           { dataType: DT.Boolean },
                isDeleted:          { dataType: DT.Boolean },
                
                room:               { dataType: DT.Int64 },
                trackIds:           { dataType: DT.Undefined },
                
                speakerIds:         { dataType: DT.Undefined },
                
                updatedAt:          { dataType: DT.DateTime }
            },
            
            navigationProperties: {
                roomId: { entityTypeName: 'Room:#EventEye.Api.Models.Contexts', isScalar: true, associationName: 'Room_ScheduleItem', foreignKeyNames: ["room"] }, 
                //trackId: { entityTypeName: 'Track:#EventEye.Api.Models.Contexts', isScalar: true, associationName: 'Track_ScheduleItem', foreignKeyNames: ["track"] }, 
                tracksScalar: { entityTypeName: 'Track:#EventEye.Api.Models.Contexts', isScalar: true, associationName: 'ScheduleItem_Track', foreignKeyNames: ["trackIds"] },
                speakersScalar: { entityTypeName: 'Speaker:#EventEye.Api.Models.Contexts', isScalar: true, associationName: 'ScheduleItem_Speaker', foreignKeyNames: ["speakerIds"] }
                //speakers: { entityTypeName: 'Speaker:#EventEye.Api.Models.Contexts', isScalar: false, associationName: 'Speaker_ScheduleItem' }                
            }
            
        });
        
        
        /*
         * Track Model
         */
        
        metadataStore.addEntityType({
            shortName: "Track",
            namespace: "EventEye.Api.Models.Contexts",
            autoGeneratedKeyType: breeze.AutoGeneratedKeyType.Identity,
            dataProperties: {
                id:                 { dataType: DT.Int64, isPartOfKey: true },
                is:                 { dataType: DT.Int64 },
                
                title:              { dataType: DT.String },             
               
                eventId:            { dataType: DT.Int64 },

                isDeleted:          { dataType: DT.Boolean },
                
                scheduleItemIds:      { dataType: DT.Undefined },
                
                updatedAt:          { dataType: DT.DateTime }
            },
            
            navigationProperties: {
                //scheduleItems: { entityTypeName: 'ScheduleItem:#EventEye.Api.Models.Contexts', isScalar: false, associationName: 'Track_ScheduleItem' }     
                scheduleItemsScalar: { entityTypeName: 'ScheduleItem:#EventEye.Api.Models.Contexts', isScalar: true, associationName: 'Track_ScheduleItem', foreignKeyNames: ["id"] },
            }
            
        });
        
        
        /*
         * Room Model
         */
        
        metadataStore.addEntityType({
            shortName: "Room",
            namespace: "EventEye.Api.Models.Contexts",
            autoGeneratedKeyType: breeze.AutoGeneratedKeyType.Identity,
            dataProperties: {
                id:                 { dataType: DT.Int64, isPartOfKey: true },
                is:                 { dataType: DT.Int64 },
                
                title:              { dataType: DT.String },
                colorCode:          { dataType: DT.String },              
               
                eventId:            { dataType: DT.Int64 },
                
                isDeleted:          { dataType: DT.Boolean },
                
                scheduleItemIds:      { dataType: DT.Undefined },
                
                updatedAt:          { dataType: DT.DateTime }
            },
            
            navigationProperties: {
                //scheduleItems: { entityTypeName: 'ScheduleItem:#EventEye.Api.Models.Contexts', isScalar: false, associationName: 'Room_ScheduleItem' }     
            }
            
        }); 
        
        
        /*
         * Speaker Model
         */
        
        
        metadataStore.addEntityType({
            shortName: "Speaker",
            namespace: "EventEye.Api.Models.Contexts",
            autoGeneratedKeyType: breeze.AutoGeneratedKeyType.Identity,
            dataProperties: {
                id:                     { dataType: DT.Int64, isPartOfKey: true },
                is:                     { dataType: DT.Int64 },
                
                firstName:              { dataType: DT.String },
                lastName:               { dataType: DT.String },              
                companyName:            { dataType: DT.String },
                position:               { dataType: DT.String },
                description:            { dataType: DT.String },
                mobile:                 { dataType: DT.String },
                twitter:                { dataType: DT.String },              
                email:                  { dataType: DT.String },
                linkedIn:               { dataType: DT.String },
                imageUrl:               { dataType: DT.String },
                
                isDeleted:              { dataType: DT.Boolean },
                
                scheduleItemIds:        { dataType: DT.Int64 },
                
                updatedAt:              { dataType: DT.DateTime }
            },
            
            navigationProperties: {
                scheduleItemsScalar: { entityTypeName: 'ScheduleItem:#EventEye.Api.Models.Contexts', isScalar: true, associationName: 'Speaker_ScheduleItem', foreignKeyNames: ["id"] },
                //scheduleItems: { entityTypeName: 'ScheduleItem:#EventEye.Api.Models.Contexts', isScalar: false, associationName: 'ScheduleItem_Speaker'}
            }
            
        });
        
        
        
        /*
         * Feed - External Content Model
         */
        
        
        metadataStore.addEntityType({
            shortName: "FeedExternalContent",
            namespace: "EventEye.Api.Models.Contexts",
            autoGeneratedKeyType: breeze.AutoGeneratedKeyType.Identity,
            dataProperties: {
                id:                     { dataType: DT.Int64, isPartOfKey: true },
                is:                     { dataType: DT.Int64 },
                
                sourceType:             { dataType: DT.Int64 },
                contentType:            { dataType: DT.Int64 },
                likesCount:             { dataType: DT.Int64 },
                commentsCount:          { dataType: DT.Int64 },
                
                text:                   { dataType: DT.String },  
                fullName:               { dataType: DT.String },
                contentUrl:             { dataType: DT.String },
                userImageUrl:           { dataType: DT.String },
                
                createdAt:              { dataType: DT.DateTime },
                date:                   { dataType: DT.DateTime },           // DEPRECIATED?   
                
                isUserLiked:            { dataType: DT.Boolean },
                isDeleted:              { dataType: DT.Boolean },
                
                updatedAt:              { dataType: DT.DateTime },

                participant:            { dataType: DT.Int64 },
                exhibitor:              { dataType: DT.Int64 }
            },
            navigationProperties: {
                participantId: { entityTypeName: 'Participant:#EventEye.Api.Models.Contexts', isScalar: true, associationName: 'Participant_Post', foreignKeyNames: ["participant"] },
                exhibitorId: { entityTypeName: 'Exhibitor:#EventEye.Api.Models.Contexts', isScalar: true, associationName: 'Exhibitor_Post', foreignKeyNames: ["exhibitor"] }
            }           
            
        });
        
        /*
         * Info Page Model
         */
        

        metadataStore.addEntityType({
            shortName: "CustomPage",
            namespace: "EventEye.Api.Models.Contexts",
            autoGeneratedKeyType: breeze.AutoGeneratedKeyType.Identity,
            dataProperties: {
                id:                     { dataType: DT.Int64, isPartOfKey: true },
                is:                     { dataType: DT.Int64 },
                
                eventId:                { dataType: DT.Int64 },
                
                title:                  { dataType: DT.String },  
                menuTitle:              { dataType: DT.String },
                content:                { dataType: DT.String },
                
                type:                   { dataType: DT.Int64  },
                imageUrl:               { dataType: DT.String },
                
                isDeleted:              { dataType: DT.Boolean },
                
                updatedAt:              { dataType: DT.DateTime }
               
            }           
            
        });
        
        /*
         * Location Map Model
         */
        
        metadataStore.addEntityType({
            shortName: "LocationMap",
            namespace: "EventEye.Api.Models.Contexts",
            autoGeneratedKeyType: breeze.AutoGeneratedKeyType.Identity,
            dataProperties: {
                id:                     { dataType: DT.Int64, isPartOfKey: true },
                is:                     { dataType: DT.Int64 },
                
                markers:               { complexTypeName:'Marker:#EventEye.Api.Models.Contexts', isScalar:false },
                
                title:                  { dataType: DT.String },  
                menuTitle:              { dataType: DT.String },
                
                isDeleted:              { dataType: DT.Boolean },
                
                updatedAt:              { dataType: DT.DateTime }
            }        
            
        });

         metadataStore.addEntityType({
            shortName: "Marker",
            namespace: "EventEye.Api.Models.Contexts",
             isComplexType: true,
            dataProperties: {
                id:                     { dataType: DT.Int64, isPartOfKey: true },
                
                title:                  { dataType: DT.String },  
                description:             { dataType: DT.String },
                latitude:                { dataType: DT.Decimal },
                longitude:                { dataType: DT.Decimal },
                
                isDeleted:              { dataType: DT.Boolean },
                updatedAt:              { dataType: DT.DateTime }
            }           
            
        });
        
        /*
         * Exhibitor List
         */
        
        metadataStore.addEntityType({
            shortName: "Exhibitor",
            namespace: "EventEye.Api.Models.Contexts",
            autoGeneratedKeyType: breeze.AutoGeneratedKeyType.Identity,
            dataProperties: {
                id:                     { dataType: DT.Int64, isPartOfKey: true },
                is:                     { dataType: DT.Int64 },
                
                priority:               { dataType: DT.Int64 },
                
                companyName:            { dataType: DT.String },
                logoImageUrlThumbnail:  { dataType: DT.String },
                logoImageUrl:           { dataType: DT.String },
                location:               { dataType: DT.String },
                description:            { dataType: DT.String },
                websiteUrl:             { dataType: DT.String },
                email:                  { dataType: DT.String },
                telephone:              { dataType: DT.String },
                contactPersonName:      { dataType: DT.String },

                isDeleted:              { dataType: DT.Boolean },
                updatedAt:              { dataType: DT.DateTime}
            }
            
        });
        
        
        /*
         * Event Info
         */
        

        metadataStore.addEntityType({
            shortName: "Event",
            namespace: "EventEye.Api.Models.Contexts",
            autoGeneratedKeyType: breeze.AutoGeneratedKeyType.Identity,
            dataProperties: {
                id:                         { dataType: DT.Int64, isPartOfKey: true },
                is:                         { dataType: DT.Int64 },
                
                timeZoneOffsetInSeconds:    { dataType: DT.Int64 },

                agendaTransparencyFactor:   { dataType: DT.Decimal },
                
                title:                      { dataType: DT.String },  
                subTitle:                   { dataType: DT.String },
                hashtags:                   { dataType: DT.String },
                country:                    { dataType: DT.String },
                timeZoneIndex:              { dataType: DT.String },
                description:                { dataType: DT.String },
                authToken:                  { dataType: DT.String },
                imageUrl:                   { dataType: DT.String },
                integrationEventId:         { dataType: DT.String },   
                language:                   { dataType: DT.String },
                primaryColor:               { dataType: DT.String },
                primaryFontColor:           { dataType: DT.String },
                secondaryColor:             { dataType: DT.String },
                secondaryFontColor:         { dataType: DT.String },
                eventIconImageUrl:          { dataType: DT.String },
                eventSplashScreenImageUrl:  { dataType: DT.String },
                integrationApiUrl:          { dataType: DT.String },
                
                participantId:              { dataType: DT.Int64 },
                profileId:                  { dataType: DT.Int64 },

                startTime:                  { dataType: DT.DateTime },
                endTime:                    { dataType: DT.DateTime },  
                         
                moduleFeed:                 { dataType: DT.Boolean },
                moduleSchedule:             { dataType: DT.Boolean },
                moduleProfile:              { dataType: DT.Boolean },
                moduleParticipants:         { dataType: DT.Boolean },
                moduleSpeakers:             { dataType: DT.Boolean },
                moduleExhibitors:           { dataType: DT.Boolean },
                moduleMaps:                 { dataType: DT.Boolean },
                moduleVenueMaps:            { dataType: DT.Boolean },
                moduleInfoPages:            { dataType: DT.Boolean },
                modulePolls:                { dataType: DT.Boolean },
                moduleOpenQuestions:        { dataType: DT.Boolean },
                moduleGallery:              { dataType: DT.Boolean },
                moduleNoPoweredByMessage:   { dataType: DT.Boolean },
                autoloadEvent:              { dataType: DT.Boolean },
                allowEventList:             { dataType: DT.Boolean },
                isOpen:                     { dataType: DT.Boolean },
                allowSendPosts:             { dataType: DT.Boolean },
                allowSendComments:          { dataType: DT.Boolean },
                allowSendLikes:             { dataType: DT.Boolean },
                
                analyticsLevel:             { dataType: DT.Int64 },
                allowNonAnonymousTracking:  { dataType: DT.Boolean},
                
                isInReview:                 { dataType: DT.Boolean },
                isDeleted:                  { dataType: DT.Boolean },
                
                updatedAt:                  { dataType: DT.DateTime }
            }           
            
        });
        
        
        
        metadataStore.setEntityTypeForResourceName('participant',           'Participant');
        metadataStore.setEntityTypeForResourceName('scheduleItem',          'ScheduleItem');
        metadataStore.setEntityTypeForResourceName('track',                 'Track');
        metadataStore.setEntityTypeForResourceName('room',                  'Room');
        metadataStore.setEntityTypeForResourceName('speaker',               'Speaker');
        metadataStore.setEntityTypeForResourceName('feedExternalContent',   'FeedExternalContent');
        metadataStore.setEntityTypeForResourceName('customPage',            'CustomPage');
        metadataStore.setEntityTypeForResourceName('event',                 'Event');
        metadataStore.setEntityTypeForResourceName('maps',                  'LocationMap');
        metadataStore.setEntityTypeForResourceName('exhibitor',             'Exhibitor');
        metadataStore.setEntityTypeForResourceName('profile',               'Profile');
        
    }   

})