'use strict';

var app = angular.module('app');

app.factory('AnalyticsService', ['appConstants', '$log', 'eventService', 'ObjectService', 'datacontext', 'ImgCache', function(appConstants, $log, eventService, ObjectService, datacontext, ImgCache) {

    var analyticLevels = {
        'HTTP Error': -1,
        'App Open': -1,
        'About App Contact Form Sent': -1,
        'Signed In': 1,
        'State Open': 3,
        'Agenda Profile State Open': 2,
        'Speaker List State Open': 3,
        'Speaker Profile State Open': 2,
        'Participant List State Open': 3,
        'Participant Profile State Open': 2,
        'Exhibitor Profile State Open': 2,
        'Infopage State Open': 3,
        'Location Map State Open': 3,
        'Venue Map State Open': 3,
        'Menu State Open': 3,
        'Event List State Open': -1,
        'Open Question Answer Feed State Open': 3,
        'Open Question List State Open': 3,
        'Open Question Details State Open': 3,
        'Poll List State Open': 3,
        'Poll Details State Open': 3,
        'Compose Tweet Button Clicked': 3,
        'Login State Open': 2,
        'Enable Push Notification Pre Question Response': 2,
        'About App State Open': 2,
        'Cache': 4

    };

    var analyticsLevelSetting = function() {

        if (!appConstants.app.analytics.activated /*|| !appConstants.api.production*/ ) {

            return 0;

        } else {

            try {
                var currentEvent = ObjectService.getSimpleObject(e.apiContentTypes.Event, datacontext.getCachedEventInformation(eventService.getId()));

                return currentEvent.analyticsLevel;
            } catch (err) {
                return 0;
            }
            // Setting analyticslevel temprorarilly hardcoded. Should be fetched from eventinfo when api is updated. TODO


        }


    }

    var AnalyticsService = {

        /*registerPeopleProperties: function(event, properties) {
        console.log("Register super property function");
            if (analyticLevels[event] <= analyticsLevelSetting()) {
                try {
                    console.log("Trying to register: " + JSON.stringify(properties));
                    mixpanel.people.set(properties);
                } catch (err) {
                    $log.debug("[ANALYTICS] Mixpanel call failed for registering properties");
                }
            }
        },*/

        trackEvent: function(event, properties) {

            //console.log("event: " + event);
            //console.log("analyticLevels[event]: " + analyticLevels[event]);
            //console.log("analyticsLevelSetting: " + analyticsLevelSetting());

            if (analyticLevels[event] <= analyticsLevelSetting()) {

                try {
                    mixpanel.register_once({
                        "appId": appConstants.app.appId
                    });
                } catch (err) {
                    $log.debug("[Analytics] Mixpanel call failed for registering appId");
                }

                if (event && properties) {
                    $log.debug("[Analytics] Tracking (w/Properties) " + event);
                    properties["eventId"] = eventService.getId();
                    if (analyticsLevelSetting() >= analyticLevels["Cache"]) {
                        properties["cacheSize"] = ImgCache.getCurrentSize();
                    }                    
                    try {
                        mixpanel.track(event, properties);
                    } catch (err) {
                        $log.debug("[Analytics] Mixpanel call failed for event: " + event + " (Properties: " + JSON.stringify(properties) + ")");
                    }

                } else {
                    $log.debug("[Analytics] Tracking (w/o Properties) " + event);
                    var properties = [];
                    properties["eventId"] = eventService.getId();
                    if (analyticsLevelSetting() >= analyticLevels["Cache"]) {
                        properties["cacheSize"] = ImgCache.getCurrentSize();
                    }   
                    try {
                        mixpanel.track(event, properties);
                    } catch (err) {
                        $log.debug("[Analytics] Mixpanel call failed for event: " + event + " (Properties: " + JSON.stringify(properties) + ")");
                    }

                }

            } else {
                $log.debug("[Analytics] Tracking blocked: Analytics disabled.");
            }

        },

        startTimer: function(timerName) {
            mixpanel.time_event(timerName);

        },

    };





    return AnalyticsService;
}]);