'use strict';

var app = angular.module('app');

app.factory('FeedService', ['appConstants', 'appConfiguration', '$log', '$http', '$q', 'session', 'ObjectService', 'datacontext', function (appConstants, appConfiguration, $log, $http, $q, session, ObjectService, datacontext) {

    var FeedService = {

        postText: function (message) {

            var url = appConfiguration.apiUrl + "feed/posts";

            var formdata = new FormData();

            formdata.append('message', message);

            var request = {
                method: 'POST',
                url: url,
                data: formdata,
                headers: {
                    'Content-Type': undefined
                }
            };

            var promise = $http(request);

            return promise;

        },

        postTextImage: function (message, photoUri) {
            return $q(function (resolve, reject) {

                var success = function (r) {
                    resolve();
                }

                var failed = function (error) {
                    reject();
                }

                var fileTransferOptions = new FileUploadOptions();
                $log.debug("FileTransferOptions:" + fileTransferOptions);
                fileTransferOptions.httpMethod = "POST";
                fileTransferOptions.mimeType = "image/jpg";
                if (message.length > 0) {
                    fileTransferOptions.params = {
                        'message': message
                    };
                }
                fileTransferOptions.fileKey = "image";
                fileTransferOptions.headers = {
                    Connection: "close",
                    'X-ZUMO-AUTH': session.getEventToken()
                }
                fileTransferOptions.chunkedMode = false;

                var url = appConfiguration.apiUrl + "feed/posts";

                var trimmedImageUrl = photoUri; //for android...
                if (ionic.Platform.isIOS()) {
                    var trimmedImageUrl = photoUri.substr(7);
                }

                var ft = new FileTransfer();
                ft.upload(trimmedImageUrl, encodeURI(url), success, failed, fileTransferOptions);

            });
        },
        deletePost: function(postId) {
            var url = appConfiguration.apiUrl + "feed/posts/" + postId;

            var promise = $http.delete(url);
            promise.success(function (data, status) {
                if (status === 200) {
                    return data;
                }
            }).error(function (data, status) {
            });
            return promise;
        },
        postLike: function (postId) {

            ///api/v1.2.4/feed/posts/<postId>/likes

            var url = appConfiguration.apiUrl + "feed/posts/" + postId + "/likes";

            var promise = $http.post(url);
            promise.success(function (data, status) {
                if (status === 200) {
                    return data;
                }
            }).error(function (data, status) {
            });
            return promise;
        },
        deleteLike: function (postId) {

            var url = appConfiguration.apiUrl + "feed/posts/" + postId + "/likes";

            var promise = $http.delete(url);
            promise.success(function (data, status) {
                if (status === 200) {
                    return data;
                }
            }).error(function (data, status) {
            });
            return promise;

        },
        getLikes: function(postId) {
            
                var url = appConfiguration.apiUrl + "feed/posts/" + postId + "/likes";
    
                var promise = $http.get(url);
                promise.success(function (data, status) {
                    if (status === 200) {
    
                        var results = data;     
                 
                        var i = 0;
                        for (i = 0; i < results.length; i++) {
                            var like = results[i];
                            var richLike = {};
                            var likeAuthorInformation = ObjectService.getSimpleObject(e.apiContentTypes.Participant, datacontext.getCachedParticipant(like.participantId));
                            like.participant = likeAuthorInformation;
                        }
                        
                        return results;
                    }
                }).error(function (data, status) {
                });
                return promise;
        },
        postComment: function(postId, text) {
            var url = appConfiguration.apiUrl + "feed/posts/" + postId + "/comments";

            var payload = {
                'text': text
            };

            var promise = $http.post(url, '"' + text + '"');
            promise.success(function (data, status) {
                if (status === 200) {
                    return data;
                }
            }).error(function (data, status) {
            });
            return promise;
        },
        getComments: function(postId) {
            var url = appConfiguration.apiUrl + "feed/posts/" + postId + "/comments";

            var promise = $http.get(url);
            promise.success(function (data, status) {
                if (status === 200) {

                    var results = data;     
             
                    //var richComments = [];
                    var i = 0;
                    for (i = 0; i < results.length; i++) {
                        var comment = results[i];
                        var richComment = {};
                        var commentAuthorInformation = ObjectService.getSimpleObject(e.apiContentTypes.Participant, datacontext.getCachedParticipant(comment.participantId));
                        comment.participant = commentAuthorInformation;
                    }
                    
                    return results;
                }
            }).error(function (data, status) {
            });
            return promise;
        },
        deleteComment: function(postId, commentId) {
            var url = appConfiguration.apiUrl + "feed/posts/" + postId + "/comments/" + commentId;

            var promise = $http.delete(url);
            promise.success(function (data, status) {
                if (status === 200) {
                    return data;
                }
            }).error(function (data, status) {
            });
            return promise;
        },

    };

    return FeedService;
}]);