'use strict';

var app = angular.module('app');

app.factory('PollService', ['$window', '$http', 'appConstants', 'appConfiguration', '$log', function ($window, $http, appConstants, appConfiguration, $log) {

    var PollService = {

        getPolls: function () {

            $log.debug("Getting polls from server");

            var url = appConfiguration.apiUrl + "poll";

            var promise = $http.get(url);
            promise.success(function (data, status) {
                if (status === 200) {
                    $log.debug("Fetching Poll SUCCESS: " + data + " " + status);
                    
                    /*
                    var seen = []
                    var json = JSON.stringify(data, function (key, val) {
                        if (typeof val == "object") {
                            if (seen.indexOf(val) >= 0)
                                return
                            seen.push(val)
                        }
                        return val
                    }, 0)

                    $log.debug(json);*/
                    
                }
            }).error(function (data, status) {
                $log.debug("Fetching Poll ERROR: " + data + " " + status);
            });
            return promise;
        },
        getPollDetails: function (pollId) {
            $log.debug("Getting poll from server");

            var url = appConfiguration.apiUrl + "poll/" + pollId;

            var promise = $http.get(url);
            promise.success(function (data, status) {
                if (status === 200) {
                    $log.debug("Fetching Poll SUCCESS: " + data + " " + status);
                    return data;
                    /*
                    var seen = []
                    var json = JSON.stringify(data, function (key, val) {
                        if (typeof val == "object") {
                            if (seen.indexOf(val) >= 0)
                                return
                            seen.push(val)
                        }
                        return val
                    }, 0)

                    $log.debug(json);*/
                    
                }
            }).error(function (data, status) {
                $log.debug("Fetching Poll ERROR: " + data + " " + status);
            });
            return promise;
        },
        putVote: function(pollId, alternativeId) {
            $log.debug("UPdating poll: " + pollId + " / " + alternativeId);

            var url = appConfiguration.apiUrl + "poll/" + pollId + "/vote/" + alternativeId;

            var promise = $http.put(url);
            promise.success(function (data, status) {
                if (status === 200) {
                    $log.debug("Updating Poll Answer SUCCESS: " + data + " " + status);
                    return data;   
                }
            }).error(function (data, status) {
                $log.debug("Updating Poll Answer ERROR: " + data + " " + status);
            });
            return promise;
        },
        
        getOpenQuestions: function() {
            $log.debug("Getting open questionss from server");

            var url = appConfiguration.apiUrl + "open-questions";

            var promise = $http.get(url);
            promise.success(function (data, status) {
                if (status === 200) {
                    $log.debug("Fetching Open Questions SUCCESS: " + data + " " + status);
                    
                    /*
                    var seen = []
                    var json = JSON.stringify(data, function (key, val) {
                        if (typeof val == "object") {
                            if (seen.indexOf(val) >= 0)
                                return
                            seen.push(val)
                        }
                        return val
                    }, 0)

                    $log.debug(json);*/
                    
                }
            }).error(function (data, status) {
                $log.debug("Fetching open question ERROR: " + data + " " + status);
            });
            return promise;
        },
        getOpenQuestionDetails: function(openQuestionId) {
            $log.debug("Getting open question from server: " + openQuestionId);

            var url = appConfiguration.apiUrl + "open-questions/" + openQuestionId;

            var promise = $http.get(url);
            promise.success(function (data, status) {
                if (status === 200) {
                    $log.debug("Fetching open question SUCCESS: " + data + " " + status);
                    return data;
                    /*
                    var seen = []
                    var json = JSON.stringify(data, function (key, val) {
                        if (typeof val == "object") {
                            if (seen.indexOf(val) >= 0)
                                return
                            seen.push(val)
                        }
                        return val
                    }, 0)

                    $log.debug(json);*/
                    
                }
            }).error(function (data, status) {
                $log.debug("Fetching open question ERROR: " + data + " " + status);
            });
            return promise;
        },
        getOpenQuestionNewAnswersCount: function(openQuestionId, revisionDate) {
            $log.debug("Getting open question new answers count from server");

            var url = appConfiguration.apiUrl + "open-questions/" + openQuestionId + "/new-answers?after=" + revisionDate;

            var promise = $http.get(url);
            promise.success(function (data, status) {
                if (status === 200) {
                    $log.debug("Fetching open question new answers SUCCESS: " + JSON.stringify(data) + " " + status);
                    return data;
                    /*
                    var seen = []
                    var json = JSON.stringify(data, function (key, val) {
                        if (typeof val == "object") {
                            if (seen.indexOf(val) >= 0)
                                return
                            seen.push(val)
                        }
                        return val
                    }, 0)

                    $log.debug(json);*/
                    
                }
            }).error(function (data, status) {
                $log.debug("Fetching open question new answers ERROR: " + data + " " + status);
            });
            return promise;
        },
        postOpenQuestionAnswer: function(openQuestionId, answerText) {
            $log.debug("Posting Open Question: " + openQuestionId + " " + answerText);

            var url = appConfiguration.apiUrl + "open-questions/" + openQuestionId + "/answers";
            //console.log(url);
            
            var promise = $http.post(url, '"' + answerText + '"');
            promise.success(function (data, status) {
                if (status === 200) {
                    $log.debug("Posting Open Question Answer SUCCESS: " + data + " " + status);
                    return data;   
                }
            }).error(function (data, status) {
                $log.debug("Posting Open Question Answer ERROR: " + data + " " + status);
            });
            return promise;
        }
    };



    return PollService;
}]);