'use strict';

var app = angular.module('app');

app.service('PushService', ['$window', '$http', '$state', 'appConstants', 'appConfiguration', '$q', '$timeout', '$log', '$ionicPopup', '$ionicPlatform',
    function($window, $http, $state, appConstants, appConfiguration, $q, $timeout, $log, $ionicPopup, $ionicPlatform) {

        var pushService = {};

        var regId = "";

        pushService.registerDevice = function() {

            $log.debug("Registering for Push Notifications");

            var push = PushNotification.init({
                "android": {
                    senderID: appConstants.push.gcmProjectId
                },
                "ios": {
                    alert: "true",
                    badge: "true",
                    sound: "true"
                },
                "windows": {}
            });

            push.on('registration', function(data) {
                regId = data.registrationId;
                if (ionic.Platform.isIOS()) {
                    pushService.storeDeviceToken("apns");
                } else if (ionic.Platform.isAndroid()) {
                    pushService.storeDeviceToken("gcm");
                }
            });

            push.on('notification', function(data) {
                
				// iOS notification called upon notification received with data: {"message":"Test Message","additionalData":{"InAppMessageTitle":"Test Title","InAppMessageStyle":4,"foreground":true,"coldstart":false,"InAppMessageText":"Test Message"}}
				// Android notification: {"title":"Title Test","message":"Message test","additionalData":{"text":"Message test","style":"4","google.message_id":"0:1489579560393262%be65a64bf9fd7ecd","foreground":true}}
                
				var notificationDisplayData = {};

                if (ionic.Platform.isAndroid()) {

                    notificationDisplayData = {
                        "title": data.title,
                        "text": data.message
                    }
                    showAlertNotification(notificationDisplayData);

                } else if (ionic.Platform.isIOS()) {
                    notificationDisplayData = {
                        "title": data.additionalData.InAppMessageTitle,
                        "text": data.additionalData.InAppMessageText
                    }
                    showAlertNotification(notificationDisplayData);
                }

            });

            push.on('error', function(e) {
                $log.debug("Push error: " + e.message); // e.message
            });

            function showAlertNotification(notification) {
                $log.debug("showAlertNotification Called");

                var alertPopup = $ionicPopup.alert({
                    cssClass: 'notificationPopup',
                    title: notification.title,
                    template: notification.text
                });

                alertPopup.then(function(res) {

                });

            }

        };

        // Register device token with EventEye Services
        pushService.storeDeviceToken = function(type) {
            $log.debug("store token called");
            var registrationIdPromise = pushService.retrieveOrRequestRegistrationIdWithDeviceToken().success(function(registrationId) {
                $log.debug("registrationIdPromise received: " + registrationId);
                var tags = [];
                var deviceInformation = {
                    Platform: type,
                    Handle: regId,
                    Tags: tags
                };

                $http.put((appConfiguration.apiUrl + "push/" + registrationId), deviceInformation)
                    .success(function(data, status) {
                        $log.debug("Token stored, device is successfully subscribed to receive push notifications.");
                    })
                    .error(function(data, status) {
                        $log.debug("Error storing device token to EventEye Services." + data + " " + status);
                    });


            });


        };

        pushService.retrieveOrRequestRegistrationIdWithDeviceToken = function() {

            function getSuccessFakePromise(dataToResolve) {

                var deferred = $q.defer()

                $timeout(function() {
                    deferred.resolve(dataToResolve);
                }, 1)
                return deferred.promise;
            }

            var registrationId = $window.localStorage.getItem(appConfiguration.localStorageKeys.push.registrationIdKey);
            /*
            if (registrationId) {
            	$log.debug("Found registrationId: " + registrationId);
            	return getSuccessFakePromise(registrationId);
            }*/

            $log.debug("retrieve or request registrationid called");
            return $http.post((appConfiguration.apiUrl + "push/"), {}, {
                    params: {
                        handle: regId
                    }
                })
                .success(function(data, status) {
                    $log.debug("Requesting registration id from EventEye Services success");
                    $window.localStorage.setItem(appConfiguration.localStorageKeys.push.registrationIdKey, data);
                    return data;

                })
                .error(function(data, status) {
                    $log.debug("Error requesting registration Id from EventEye Services" + data + " " + status);
                    return;
                });

        };
        //});

        return pushService;

    }
]);